@@ -15,8 +15,9 @@
  * @category   Zend
  * @package    Zend_Cache
  * @subpackage Zend_Cache_Frontend
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 
@@ -29,7 +30,7 @@ require_once 'Zend/Cache/Core.php';
 /**
  * @package    Zend_Cache
  * @subpackage Zend_Cache_Frontend
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Cache_Frontend_Output extends Zend_Cache_Core
@@ -54,7 +55,7 @@ class Zend_Cache_Frontend_Output extends Zend_Cache_Core
      *
      * @param  string  $id                     Cache id
      * @param  boolean $doNotTestCacheValidity If set to true, the cache validity won't be tested
-     * @param  boolean $echoData               If set to true, datas are sent to the browser if the cache is hit (simpy returned else)
+     * @param  boolean $echoData               If set to true, datas are sent to the browser if the cache is hit (simply returned else)
      * @return mixed True if the cache is hit (false else) with $echoData=true (default) ; string else (datas)
      */
     public function start($id, $doNotTestCacheValidity = false, $echoData = true)
@@ -81,21 +82,21 @@ class Zend_Cache_Frontend_Output extends Zend_Cache_Core
      * @param  int     $specificLifetime If != false, set a specific lifetime for this cache record (null => infinite lifetime)
      * @param  string  $forcedDatas      If not null, force written datas with this
      * @param  boolean $echoData         If set to true, datas are sent to the browser
+     * @param  int     $priority         integer between 0 (very low priority) and 10 (maximum priority) used by some particular backends
      * @return void
      */
-    public function end($tags = array(), $specificLifetime = false, $forcedDatas = null, $echoData = true)
+    public function end($tags = array(), $specificLifetime = false, $forcedDatas = null, $echoData = true, $priority = 8)
     {
-        if (is_null($forcedDatas)) {
-            $data = ob_get_contents();
-            ob_end_clean();
+        if ($forcedDatas === null) {
+            $data = ob_get_clean();
         } else {
             $data =& $forcedDatas;
         }
         $id = array_pop($this->_idStack);
-        if (is_null($id)) {
+        if ($id === null) {
             Zend_Cache::throwException('use of end() without a start()');
         }
-        $this->save($data, $id, $tags, $specificLifetime);
+        $this->save($data, $id, $tags, $specificLifetime, $priority);
         if ($echoData) {
             echo($data);
         }
