@@ -15,16 +15,11 @@
  * @category   Zend
  * @package    Zend_Controller
  * @subpackage Zend_Controller_Action
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
-
-/**
- * @see Zend_Controller_Action_Exception
- */
-require_once 'Zend/Controller/Action/Exception.php';
-
 /**
  * @see Zend_Controller_Action_HelperBroker_PriorityStack
  */
@@ -39,28 +34,11 @@ require_once 'Zend/Loader.php';
  * @category   Zend
  * @package    Zend_Controller
  * @subpackage Zend_Controller_Action
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Controller_Action_HelperBroker
 {
-    /**
-     * $_helpers - Helper array
-     *
-     * @var Zend_Controller_Action_HelperBroker_PriorityStack
-     */
-    static protected $_stack = null;
-
-    /**
-     * $_paths - paths to Action_Helpers
-     *
-     * @var array
-     */
-    static protected $_paths = array(array(
-        'dir'    => 'Zend/Controller/Action/Helper/',
-        'prefix' => 'Zend_Controller_Action_Helper_'
-    ));
-
     /**
      * $_actionController - ActionController reference
      *
@@ -69,15 +47,46 @@ class Zend_Controller_Action_HelperBroker
     protected $_actionController;
 
     /**
-     * addHelper() - Add helper objects
+     * @var Zend_Loader_PluginLoader_Interface
+     */
+    protected static $_pluginLoader;
+
+    /**
+     * $_helpers - Helper array
      *
-     * @param Zend_Controller_Action_Helper_Abstract $helper
+     * @var Zend_Controller_Action_HelperBroker_PriorityStack
+     */
+    protected static $_stack = null;
+
+    /**
+     * Set PluginLoader for use with broker
+     *
+     * @param  Zend_Loader_PluginLoader_Interface $loader
      * @return void
      */
-    static public function addHelper(Zend_Controller_Action_Helper_Abstract $helper)
+    public static function setPluginLoader($loader)
     {
-        self::getStack()->push($helper);
-        return;
+        if ((null !== $loader) && (!$loader instanceof Zend_Loader_PluginLoader_Interface)) {
+            require_once 'Zend/Controller/Action/Exception.php';
+            throw new Zend_Controller_Action_Exception('Invalid plugin loader provided to HelperBroker');
+        }
+        self::$_pluginLoader = $loader;
+    }
+
+    /**
+     * Retrieve PluginLoader
+     *
+     * @return Zend_Loader_PluginLoader
+     */
+    public static function getPluginLoader()
+    {
+        if (null === self::$_pluginLoader) {
+            require_once 'Zend/Loader/PluginLoader.php';
+            self::$_pluginLoader = new Zend_Loader_PluginLoader(array(
+                'Zend_Controller_Action_Helper' => 'Zend/Controller/Action/Helper/',
+            ));
+        }
+        return self::$_pluginLoader;
     }
 
     /**
@@ -88,8 +97,31 @@ class Zend_Controller_Action_HelperBroker
     static public function addPrefix($prefix)
     {
         $prefix = rtrim($prefix, '_');
-        $path = str_replace('_', DIRECTORY_SEPARATOR, $prefix);
-        self::addPath($path, $prefix);
+        $path   = str_replace('_', DIRECTORY_SEPARATOR, $prefix);
+        self::getPluginLoader()->addPrefixPath($prefix, $path);
+    }
+
+    /**
+     * addPath() - Add path to repositories where Action_Helpers could be found.
+     *
+     * @param string $path
+     * @param string $prefix Optional; defaults to 'Zend_Controller_Action_Helper'
+     * @return void
+     */
+    static public function addPath($path, $prefix = 'Zend_Controller_Action_Helper')
+    {
+        self::getPluginLoader()->addPrefixPath($prefix, $path);
+    }
+
+    /**
+     * addHelper() - Add helper objects
+     *
+     * @param Zend_Controller_Action_Helper_Abstract $helper
+     * @return void
+     */
+    static public function addHelper(Zend_Controller_Action_Helper_Abstract $helper)
+    {
+        self::getStack()->push($helper);
         return;
     }
 
@@ -104,117 +136,6 @@ class Zend_Controller_Action_HelperBroker
         return;
     }
 
-    /**
-     * addPath() - Add path to repositories where Action_Helpers could be found.
-     *
-     * @param string $path
-     * @param string $prefix Optional; defaults to 'Zend_Controller_Action_Helper'
-     * @return void
-     */
-    static public function addPath($path, $prefix = 'Zend_Controller_Action_Helper')
-    {
-        // make sure it ends in a PATH_SEPARATOR
-        if (substr($path, -1, 1) != DIRECTORY_SEPARATOR) {
-            $path .= DIRECTORY_SEPARATOR;
-        }
-
-        // make sure it ends in a PATH_SEPARATOR
-        $prefix = rtrim($prefix, '_') . '_';
-
-        $info['dir']    = $path;
-        $info['prefix'] = $prefix;
-
-        self::$_paths[] = $info;
-        return;
-    }
-
-    /**
-     * __construct() -
-     *
-     * @param Zend_Controller_Action $actionController
-     * @return void
-     */
-    public function __construct(Zend_Controller_Action $actionController)
-    {
-        $this->_actionController = $actionController;
-        foreach (self::getStack() as $helper) {
-            $helper->setActionController($actionController);
-            $helper->init();
-        }
-    }
-
-    /**
-     * notifyPreDispatch() - called by action controller dispatch method
-     *
-     * @return void
-     */
-    public function notifyPreDispatch()
-    {
-        foreach (self::getStack() as $helper) {
-            $helper->preDispatch();
-        }
-    }
-
-    /**
-     * notifyPostDispatch() - called by action controller dispatch method
-     *
-     * @return void
-     */
-    public function notifyPostDispatch()
-    {
-        foreach (self::getStack() as $helper) {
-            $helper->postDispatch();
-        }
-    }
-
-    /**
-     * Normalize helper name for lookups
-     *
-     * @param  string $name
-     * @return string
-     */
-    protected static function _normalizeHelperName($name)
-    {
-        if (strpos($name, '_') !== false) {
-            $name = str_replace(' ', '', ucwords(str_replace('_', ' ', $name)));
-        }
-
-        return ucfirst($name);
-    }
-
-    /**
-     * getHelper() - get helper by name
-     *
-     * @param  string $name
-     * @return Zend_Controller_Action_Helper_Abstract
-     */
-    public function getHelper($name)
-    {
-        $name = self::_normalizeHelperName($name);
-
-        $stack = self::getStack();
-        
-        if (!isset($stack->{$name})) {
-            self::_loadHelper($name);
-        }
-
-        $helper = $stack->{$name};
-
-        $initialize = false;
-        if (null === ($actionController = $helper->getActionController())) {
-            $initialize = true;
-        } elseif ($actionController !== $this->_actionController) {
-            $initialize = true;
-        }
-
-        if ($initialize) {
-            $helper->setActionController($this->_actionController)
-                ->init();
-        }
-
-        return $helper;
-    }
-
     /**
      * Retrieve or initialize a helper statically
      *
@@ -227,17 +148,14 @@ class Zend_Controller_Action_HelperBroker
      */
     public static function getStaticHelper($name)
     {
-        $name = self::_normalizeHelperName($name);
-
+        $name  = self::_normalizeHelperName($name);
         $stack = self::getStack();
-        
+
         if (!isset($stack->{$name})) {
             self::_loadHelper($name);
         }
 
-        $helper = $stack->{$name};
-
-        return $helper;
+        return $stack->{$name};
     }
 
     /**
@@ -257,15 +175,15 @@ class Zend_Controller_Action_HelperBroker
      */
     public static function getExistingHelper($name)
     {
-        $name = self::_normalizeHelperName($name);
-
+        $name  = self::_normalizeHelperName($name);
         $stack = self::getStack();
-        
-        if (isset($stack->{$name})) {
-            return $stack->{$name};
+
+        if (!isset($stack->{$name})) {
+            require_once 'Zend/Controller/Action/Exception.php';
+            throw new Zend_Controller_Action_Exception('Action helper "' . $name . '" has not been registered with the helper broker');
         }
 
-        throw new Zend_Controller_Action_Exception('Action helper "' . $name . '" has not been registered with the helper broker');
+        return $stack->{$name};
     }
 
     /**
@@ -317,57 +235,83 @@ class Zend_Controller_Action_HelperBroker
         if (self::$_stack == null) {
             self::$_stack = new Zend_Controller_Action_HelperBroker_PriorityStack();
         }
-        
+
         return self::$_stack;
     }
-    
+
     /**
-     * _loadHelper()
+     * Constructor
      *
-     * @param  string $name
+     * @param Zend_Controller_Action $actionController
      * @return void
      */
-    protected static function _loadHelper($name)
+    public function __construct(Zend_Controller_Action $actionController)
     {
-        $file = $name . '.php';
-
-        $paths = array_reverse(self::$_paths);
-        foreach ($paths as $info) {
-            $dir    = $info['dir'];
-            $prefix = $info['prefix'];
-
-            $class = $prefix . $name;
-
-            if (class_exists($class, false)) {
-                $helper = new $class();
-
-                if (!$helper instanceof Zend_Controller_Action_Helper_Abstract) {
-                    throw new Zend_Controller_Action_Exception('Helper name ' . $name . ' -> class ' . $class . ' is not of type Zend_Controller_Action_Helper_Abstract');
-                }
-
-                self::getStack()->push($helper);
-                return;
-
-            } elseif (Zend_Loader::isReadable($dir . $file)) {
-                include_once $dir . $file;
-
-                if (class_exists($class, false)) {
-                    $helper = new $class();
-                    if (!$helper instanceof Zend_Controller_Action_Helper_Abstract) {
-                        throw new Zend_Controller_Action_Exception('Helper name ' . $name . ' -> class ' . $class . ' is not of type Zend_Controller_Action_Helper_Abstract');
-                    }
-
-                    self::getStack()->push($helper);
-                    return;
-                }
-            }
+        $this->_actionController = $actionController;
+        foreach (self::getStack() as $helper) {
+            $helper->setActionController($actionController);
+            $helper->init();
         }
-
-        throw new Zend_Controller_Action_Exception('Action Helper by name ' . $name . ' not found.');
     }
 
     /**
-     * __call()
+     * notifyPreDispatch() - called by action controller dispatch method
+     *
+     * @return void
+     */
+    public function notifyPreDispatch()
+    {
+        foreach (self::getStack() as $helper) {
+            $helper->preDispatch();
+        }
+    }
+
+    /**
+     * notifyPostDispatch() - called by action controller dispatch method
+     *
+     * @return void
+     */
+    public function notifyPostDispatch()
+    {
+        foreach (self::getStack() as $helper) {
+            $helper->postDispatch();
+        }
+    }
+
+    /**
+     * getHelper() - get helper by name
+     *
+     * @param  string $name
+     * @return Zend_Controller_Action_Helper_Abstract
+     */
+    public function getHelper($name)
+    {
+        $name  = self::_normalizeHelperName($name);
+        $stack = self::getStack();
+
+        if (!isset($stack->{$name})) {
+            self::_loadHelper($name);
+        }
+
+        $helper = $stack->{$name};
+
+        $initialize = false;
+        if (null === ($actionController = $helper->getActionController())) {
+            $initialize = true;
+        } elseif ($actionController !== $this->_actionController) {
+            $initialize = true;
+        }
+
+        if ($initialize) {
+            $helper->setActionController($this->_actionController)
+                   ->init();
+        }
+
+        return $helper;
+    }
+
+    /**
+     * Method overloading
      *
      * @param  string $method
      * @param  array $args
@@ -377,15 +321,15 @@ class Zend_Controller_Action_HelperBroker
     public function __call($method, $args)
     {
         $helper = $this->getHelper($method);
-        if (method_exists($helper, 'direct')) {
-            return call_user_func_array(array($helper, 'direct'), $args);
+        if (!method_exists($helper, 'direct')) {
+            require_once 'Zend/Controller/Action/Exception.php';
+            throw new Zend_Controller_Action_Exception('Helper "' . $method . '" does not support overloading via direct()');
         }
-
-        throw new Zend_Controller_Action_Exception('Helper "' . $method . '" does not support overloading via direct()');
+        return call_user_func_array(array($helper, 'direct'), $args);
     }
 
     /**
-     * __get()
+     * Retrieve helper by name as object property
      *
      * @param  string $name
      * @return Zend_Controller_Action_Helper_Abstract
@@ -394,4 +338,44 @@ class Zend_Controller_Action_HelperBroker
     {
         return $this->getHelper($name);
     }
+
+    /**
+     * Normalize helper name for lookups
+     *
+     * @param  string $name
+     * @return string
+     */
+    protected static function _normalizeHelperName($name)
+    {
+        if (strpos($name, '_') !== false) {
+            $name = str_replace(' ', '', ucwords(str_replace('_', ' ', $name)));
+        }
+
+        return ucfirst($name);
+    }
+
+    /**
+     * Load a helper
+     *
+     * @param  string $name
+     * @return void
+     */
+    protected static function _loadHelper($name)
+    {
+        try {
+            $class = self::getPluginLoader()->load($name);
+        } catch (Zend_Loader_PluginLoader_Exception $e) {
+            require_once 'Zend/Controller/Action/Exception.php';
+            throw new Zend_Controller_Action_Exception('Action Helper by name ' . $name . ' not found', 0, $e);
+        }
+
+        $helper = new $class();
+
+        if (!$helper instanceof Zend_Controller_Action_Helper_Abstract) {
+            require_once 'Zend/Controller/Action/Exception.php';
+            throw new Zend_Controller_Action_Exception('Helper name ' . $name . ' -> class ' . $class . ' is not of type Zend_Controller_Action_Helper_Abstract');
+        }
+
+        self::getStack()->push($helper);
+    }
 }
