@@ -12,10 +12,12 @@
  * obtain it through the world-wide-web, please send an email
  * to license@zend.com so we can send you a copy immediately.
  *
+ * @category   Zend
  * @package    Zend_Controller
  * @subpackage Dispatcher
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 /**
@@ -31,7 +33,7 @@ require_once 'Zend/Controller/Response/Abstract.php';
 /**
  * @package    Zend_Controller
  * @subpackage Dispatcher
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 interface Zend_Controller_Dispatcher_Interface
@@ -47,6 +49,17 @@ interface Zend_Controller_Dispatcher_Interface
      */
     public function formatControllerName($unformatted);
 
+    /**
+     * Formats a string into a module name.  This is used to take a raw
+     * module name, such as one that would be packaged inside a request
+     * object, and reformat it to a proper directory/class name that a class extending
+     * Zend_Controller_Action would use.
+     *
+     * @param string $unformatted
+     * @return string
+     */
+    public function formatModuleName($unformatted);
+
     /**
      * Formats a string into an action name.  This is used to take a raw
      * action name, such as one that would be packaged inside a request
@@ -158,7 +171,7 @@ interface Zend_Controller_Dispatcher_Interface
      *
      * @param  Zend_Controller_Request_Abstract $request
      * @param  Zend_Controller_Response_Abstract $response
-     * @return Zend_Controller_Request_Abstract|boolean
+     * @return void
      */
     public function dispatch(Zend_Controller_Request_Abstract $request, Zend_Controller_Response_Abstract $response);
 
@@ -172,21 +185,21 @@ interface Zend_Controller_Dispatcher_Interface
 
     /**
      * Retrieve the default module name
-     * 
+     *
      * @return string
      */
     public function getDefaultModule();
 
     /**
      * Retrieve the default controller name
-     * 
+     *
      * @return string
      */
     public function getDefaultControllerName();
 
     /**
      * Retrieve the default action
-     * 
+     *
      * @return string
      */
     public function getDefaultAction();
