@@ -14,14 +14,15 @@
  *
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 /**
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 abstract class Zend_Controller_Request_Abstract
@@ -149,6 +150,12 @@ abstract class Zend_Controller_Request_Abstract
     public function setActionName($value)
     {
         $this->_action = $value;
+        /**
+         * @see ZF-3465
+         */
+        if (null === $value) {
+            $this->setParam($this->getActionKey(), $value);
+        }
         return $this;
     }
 
@@ -305,7 +312,7 @@ abstract class Zend_Controller_Request_Abstract
     {
         $this->_params = $this->_params + (array) $array;
 
-        foreach ($this->_params as $key => $value) {
+        foreach ($array as $key => $value) {
             if (null === $value) {
                 unset($this->_params[$key]);
             }
@@ -314,6 +321,17 @@ abstract class Zend_Controller_Request_Abstract
         return $this;
     }
 
+    /**
+     * Unset all user parameters
+     *
+     * @return Zend_Controller_Request_Abstract
+     */
+    public function clearParams()
+    {
+        $this->_params = array();
+        return $this;
+    }
+
     /**
      * Set flag indicating whether or not request has been dispatched
      *
