@@ -15,8 +15,8 @@
  * @category   Zend
  * @package    Zend_Controller
  * @subpackage Router
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
- * @version    $Id: Route.php 16541 2009-07-07 06:59:03Z bkarwin $
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
+ * @version    $Id$
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -28,7 +28,7 @@ require_once 'Zend/Controller/Router/Route/Abstract.php';
  *
  * @package    Zend_Controller
  * @subpackage Router
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  * @see        http://manuals.rubyonrails.com/read/chapter/65
  */
@@ -47,28 +47,28 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
      * @var Zend_Translate
      */
     protected $_translator;
-    
+
     /**
      * Default locale
      *
      * @var mixed
      */
     protected static $_defaultLocale;
-    
+
     /**
      * Locale
-     * 
+     *
      * @var mixed
      */
     protected $_locale;
-    
+
     /**
      * Wether this is a translated route or not
      *
      * @var boolean
      */
     protected $_isTranslated = false;
-    
+
     /**
      * Translatable variables
      *
@@ -77,7 +77,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     protected $_translatable = array();
 
     protected $_urlVariable = ':';
-    protected $_urlDelimiter = '/';
+    protected $_urlDelimiter = self::URI_DELIMITER;
     protected $_regexDelimiter = '#';
     protected $_defaultRegex = null;
 
@@ -132,7 +132,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     public function getVersion() {
         return 1;
     }
-    
+
     /**
      * Instantiates route based on passed Zend_Config structure
      *
@@ -153,7 +153,6 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
      * @param string $route Map used to match with later submitted URL path
      * @param array $defaults Defaults for map variables with keys as variable names
      * @param array $reqs Regular expression requirements for variables (keys as variable names)
-     * @param array          $reqs       Regular expression requirements for variables (keys as variable names)
      * @param Zend_Translate $translator Translator to use for this instance
      */
     public function __construct($route, $defaults = array(), $reqs = array(), Zend_Translate $translator = null, $locale = null)
@@ -174,20 +173,20 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                         $this->_translatable[] = $name;
                         $this->_isTranslated   = true;
                     }
-                    
+
                     $this->_parts[$pos]     = (isset($reqs[$name]) ? $reqs[$name] : $this->_defaultRegex);
                     $this->_variables[$pos] = $name;
                 } else {
                     if (substr($part, 0, 1) == $this->_urlVariable) {
                         $part = substr($part, 1);
                     }
-                    
+
                     if (substr($part, 0, 1) === '@' && substr($part, 1, 1) !== '@') {
                         $this->_isTranslated = true;
                     }
-                    
+
                     $this->_parts[$pos] = $part;
-                    
+
                     if ($part !== '*') {
                         $this->_staticCount++;
                     }
@@ -208,15 +207,15 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
         if ($this->_isTranslated) {
             $translateMessages = $this->getTranslator()->getMessages();
         }
-        
+
         $pathStaticCount = 0;
         $values          = array();
         $matchedPath     = '';
-        
+
         if (!$partial) {
             $path = trim($path, $this->_urlDelimiter);
         }
-        
+
         if ($path !== '') {
             $path = explode($this->_urlDelimiter, $path);
 
@@ -229,9 +228,9 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                         return false;
                     }
                 }
-                
+
                 $matchedPath .= $pathPart . $this->_urlDelimiter;
-                
+
                 // If it's a wildcard, get the rest of URL as wildcard data and stop matching
                 if ($this->_parts[$pos] == '*') {
                     $count = count($path);
@@ -241,6 +240,8 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                             $this->_wildcardData[$var] = (isset($path[$i+1])) ? urldecode($path[$i+1]) : null;
                         }
                     }
+
+                    $matchedPath = implode($this->_urlDelimiter, $path);
                     break;
                 }
 
@@ -253,12 +254,12 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                     if (substr($part, 0, 1) === '@') {
                         $part = substr($part, 1);
                     }
-                    
+
                     if (($originalPathPart = array_search($pathPart, $translateMessages)) !== false) {
                         $pathPart = $originalPathPart;
                     }
                 }
-                
+
                 if (substr($part, 0, 2) === '@@') {
                     $part = substr($part, 1);
                 }
@@ -278,7 +279,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                     $values[$name] = $pathPart;
                 } else {
                     $pathStaticCount++;
-                }  
+                }
             }
         }
 
@@ -293,13 +294,16 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
         foreach ($this->_variables as $var) {
             if (!array_key_exists($var, $return)) {
                 return false;
+            } elseif ($return[$var] == '' || $return[$var] === null) {
+                // Empty variable? Replace with the default value.
+                $return[$var] = $this->_defaults[$var];
             }
         }
-        
+
         $this->setMatchedPath(rtrim($matchedPath, $this->_urlDelimiter));
 
         $this->_values = $values;
-        
+
         return $return;
 
     }
@@ -315,7 +319,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     {
         if ($this->_isTranslated) {
             $translator = $this->getTranslator();
-            
+
             if (isset($data['@locale'])) {
                 $locale = $data['@locale'];
                 unset($data['@locale']);
@@ -323,7 +327,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                 $locale = $this->getLocale();
             }
         }
-        
+
         $url  = array();
         $flag = false;
 
@@ -343,18 +347,18 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                     $value = $this->_values[$name];
                 } elseif (!$reset && !$useDefault && isset($this->_wildcardData[$name])) {
                     $value = $this->_wildcardData[$name];
-                } elseif (isset($this->_defaults[$name])) {
+                } elseif (array_key_exists($name, $this->_defaults)) {
                     $value = $this->_defaults[$name];
                 } else {
                     require_once 'Zend/Controller/Router/Exception.php';
                     throw new Zend_Controller_Router_Exception($name . ' is not specified');
                 }
-                
+
                 if ($this->_isTranslated && in_array($name, $this->_translatable)) {
                     $url[$key] = $translator->translate($value, $locale);
                 } else {
                     $url[$key] = $value;
-                } 
+                }
             } elseif ($part != '*') {
                 if ($this->_isTranslated && substr($part, 0, 1) === '@') {
                     if (substr($part, 1, 1) !== '@') {
@@ -366,13 +370,14 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
                     if (substr($part, 0, 2) === '@@') {
                         $part = substr($part, 1);
                     }
-                    
+
                     $url[$key] = $part;
                 }
             } else {
                 if (!$reset) $data += $this->_wildcardData;
+                $defaults = $this->getDefaults();
                 foreach ($data as $var => $value) {
-                    if ($value !== null) {
+                    if ($value !== null && (!isset($defaults[$var]) || $value != $defaults[$var])) {
                         $url[$key++] = $var;
                         $url[$key++] = $value;
                         $flag = true;
@@ -385,15 +390,15 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
 
         foreach (array_reverse($url, true) as $key => $value) {
             $defaultValue = null;
-            
+
             if (isset($this->_variables[$key])) {
                 $defaultValue = $this->getDefault($this->_variables[$key]);
-               
+
                 if ($this->_isTranslated && $defaultValue !== null && isset($this->_translatable[$this->_variables[$key]])) {
                     $defaultValue = $translator->translate($defaultValue, $locale);
                 }
             }
-                
+
             if ($flag || $value !== $defaultValue || $partial) {
                 if ($encode) $value = urlencode($value);
                 $return = $this->_urlDelimiter . $value . $return;
@@ -426,7 +431,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     public function getDefaults() {
         return $this->_defaults;
     }
-    
+
     /**
      * Get all variables which are used by the route
      *
@@ -439,7 +444,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
 
     /**
      * Set a default translator
-     * 
+     *
      * @param  Zend_Translate $translator
      * @return void
      */
@@ -447,7 +452,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     {
         self::$_defaultTranslator = $translator;
     }
-    
+
     /**
      * Get the default translator
      *
@@ -457,10 +462,10 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     {
         return self::$_defaultTranslator;
     }
-    
+
     /**
      * Set a translator
-     * 
+     *
      * @param  Zend_Translate $translator
      * @return void
      */
@@ -468,10 +473,10 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     {
         $this->_translator = $translator;
     }
-    
+
     /**
      * Get the translator
-     * 
+     *
      * @throws Zend_Controller_Router_Exception When no translator can be found
      * @return Zend_Translate
      */
@@ -487,19 +492,19 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
             } catch (Zend_Exception $e) {
                 $translator = null;
             }
-            
-            if ($translator instanceof Zend_Translate) { 
+
+            if ($translator instanceof Zend_Translate) {
                 return $translator;
             }
         }
-        
+
         require_once 'Zend/Controller/Router/Exception.php';
         throw new Zend_Controller_Router_Exception('Could not find a translator');
     }
-    
+
     /**
      * Set a default locale
-     * 
+     *
      * @param  mixed $locale
      * @return void
      */
@@ -507,7 +512,7 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     {
         self::$_defaultLocale = $locale;
     }
-    
+
     /**
      * Get the default locale
      *
@@ -517,10 +522,10 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     {
         return self::$_defaultLocale;
     }
-    
+
     /**
      * Set a locale
-     * 
+     *
      * @param  mixed $locale
      * @return void
      */
@@ -528,10 +533,10 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
     {
         $this->_locale = $locale;
     }
-    
+
     /**
      * Get the locale
-     * 
+     *
      * @return mixed
      */
     public function getLocale()
@@ -546,12 +551,12 @@ class Zend_Controller_Router_Route extends Zend_Controller_Router_Route_Abstract
             } catch (Zend_Exception $e) {
                 $locale = null;
             }
-            
-            if ($locale !== null) { 
+
+            if ($locale !== null) {
                 return $locale;
             }
         }
-        
+
         return null;
     }
 }
