@@ -12,10 +12,11 @@
  * obtain it through the world-wide-web, please send an email
  * to license@zend.com so we can send you a copy immediately.
  *
+ * @category   Zend
  * @package    Zend_Controller
  * @subpackage Router
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
- * @version    $Id: Regex.php 10744 2008-08-07 02:32:44Z matthew $
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
+ * @version    $Id$
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -27,7 +28,7 @@ require_once 'Zend/Controller/Router/Route/Abstract.php';
  *
  * @package    Zend_Controller
  * @subpackage Router
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Abstract
@@ -53,29 +54,41 @@ class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Ab
 
     public function __construct($route, $defaults = array(), $map = array(), $reverse = null)
     {
-        $this->_regex = '#^' . $route . '$#i';
+        $this->_regex    = $route;
         $this->_defaults = (array) $defaults;
-        $this->_map = (array) $map;
-        $this->_reverse = $reverse;
+        $this->_map      = (array) $map;
+        $this->_reverse  = $reverse;
     }
 
     public function getVersion() {
         return 1;
     }
-    
+
     /**
      * Matches a user submitted path with a previously defined route.
      * Assigns and returns an array of defaults on a successful match.
      *
-     * @param string $path Path used to match against this routing map
-     * @return array|false An array of assigned values or a false on a mismatch
+     * @param  string $path Path used to match against this routing map
+     * @return array|false  An array of assigned values or a false on a mismatch
      */
-    public function match($path)
+    public function match($path, $partial = false)
     {
-        $path = trim(urldecode($path), '/');
-        $res = preg_match($this->_regex, $path, $values);
+        if (!$partial) {
+            $path = trim(urldecode($path), self::URI_DELIMITER);
+            $regex = '#^' . $this->_regex . '$#i';
+        } else {
+            $regex = '#^' . $this->_regex . '#i';
+        }
 
-        if ($res === 0) return false;
+        $res = preg_match($regex, $path, $values);
+
+        if ($res === 0) {
+            return false;
+        }
+
+        if ($partial) {
+            $this->setMatchedPath($values[0]);
+        }
 
         // array_filter_key()? Why isn't this in a standard PHP function set yet? :)
         foreach ($values as $i => $value) {
@@ -86,10 +99,9 @@ class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Ab
 
         $this->_values = $values;
 
-        $values = $this->_getMappedValues($values);
+        $values   = $this->_getMappedValues($values);
         $defaults = $this->_getMappedValues($this->_defaults, false, true);
-
-        $return = $values + $defaults;
+        $return   = $values + $defaults;
 
         return $return;
     }
@@ -103,10 +115,10 @@ class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Ab
      * indexed numerically then every associative key will be stripped. Vice versa if reversed
      * is set to true.
      *
-     * @param array $values Indexed or associative array of values to map
-     * @param boolean $reversed False means translation of index to association. True means reverse.
-     * @param boolean $preserve Should wrong type of keys be preserved or stripped.
-     * @return array An array of mapped values
+     * @param  array   $values Indexed or associative array of values to map
+     * @param  boolean $reversed False means translation of index to association. True means reverse.
+     * @param  boolean $preserve Should wrong type of keys be preserved or stripped.
+     * @return array   An array of mapped values
      */
     protected function _getMappedValues($values, $reversed = false, $preserve = false)
     {
@@ -125,7 +137,14 @@ class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Ab
                 }
                 $return[$index] = $values[$key];
             } elseif ($reversed) {
-                $index = (!is_int($key)) ? array_search($key, $this->_map, true) : $key;
+                $index = $key;
+                if (!is_int($key)) {
+                    if (array_key_exists($key, $this->_map)) {
+                        $index = $this->_map[$key];
+                    } else {
+                        $index = array_search($key, $this->_map, true);
+                    }
+                }
                 if (false !== $index) {
                     $return[$index] = $values[$key];
                 }
@@ -140,10 +159,10 @@ class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Ab
     /**
      * Assembles a URL path defined by this route
      *
-     * @param array $data An array of name (or index) and value pairs used as parameters
+     * @param  array $data An array of name (or index) and value pairs used as parameters
      * @return string Route path with user submitted parameters
      */
-    public function assemble($data = array(), $reset = false, $encode = false)
+    public function assemble($data = array(), $reset = false, $encode = false, $partial = false)
     {
         if ($this->_reverse === null) {
             require_once 'Zend/Controller/Router/Exception.php';
@@ -169,6 +188,12 @@ class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Ab
         $mergedData = $this->_arrayMergeNumericKeys($mergedData, $matchedValuesMapped);
         $mergedData = $this->_arrayMergeNumericKeys($mergedData, $dataValuesMapped);
 
+        if ($encode) {
+            foreach ($mergedData as $key => &$value) {
+                $value = urlencode($value);
+            }
+        }
+
         ksort($mergedData);
 
         $return = @vsprintf($this->_reverse, $mergedData);
@@ -203,6 +228,26 @@ class Zend_Controller_Router_Route_Regex extends Zend_Controller_Router_Route_Ab
         return $this->_defaults;
     }
 
+    /**
+     * Get all variables which are used by the route
+     *
+     * @return array
+     */
+    public function getVariables()
+    {
+        $variables = array();
+
+        foreach ($this->_map as $key => $value) {
+            if (is_numeric($key)) {
+                $variables[] = $value;
+            } else {
+                $variables[] = $key;
+            }
+        }
+
+        return $variables;
+    }
+
     /**
      * _arrayMergeNumericKeys() - allows for a strict key (numeric's included) array_merge.
      * php's array_merge() lacks the ability to merge with numeric keys.
