@@ -15,8 +15,10 @@
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Media
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 /**
@@ -29,13 +31,14 @@ require_once 'Zend/Gdata/App/Extension.php';
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Media
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Gdata_Media_Extension_MediaThumbnail extends Zend_Gdata_Extension
 {
 
-    protected $_rootElement = 'thumbnail'; 
+    protected $_rootElement = 'thumbnail';
     protected $_rootNamespace = 'media';
 
     /**
@@ -69,9 +72,7 @@ class Zend_Gdata_Media_Extension_MediaThumbnail extends Zend_Gdata_Extension
     public function __construct($url = null, $width = null, $height = null,
             $time = null)
     {
-        foreach (Zend_Gdata_Media::$namespaces as $nsPrefix => $nsUri) {
-            $this->registerNamespace($nsPrefix, $nsUri);
-        }
+        $this->registerAllNamespaces(Zend_Gdata_Media::$namespaces);
         parent::__construct();
         $this->_url = $url;
         $this->_width = $width;
@@ -79,29 +80,29 @@ class Zend_Gdata_Media_Extension_MediaThumbnail extends Zend_Gdata_Extension
         $this->_time = $time ;
     }
 
-    /** 
-     * Retrieves a DOMElement which corresponds to this element and all 
+    /**
+     * Retrieves a DOMElement which corresponds to this element and all
      * child properties.  This is used to build an entry back into a DOM
      * and eventually XML text for sending to the server upon updates, or
-     * for application storage/persistence.  
+     * for application storage/persistence.
      *
      * @param DOMDocument $doc The DOMDocument used to construct DOMElements
-     * @return DOMElement The DOMElement representing this element and all 
+     * @return DOMElement The DOMElement representing this element and all
      * child properties.
      */
-    public function getDOM($doc = null)
+    public function getDOM($doc = null, $majorVersion = 1, $minorVersion = null)
     {
-        $element = parent::getDOM($doc);
-        if ($this->_url != null) {
+        $element = parent::getDOM($doc, $majorVersion, $minorVersion);
+        if ($this->_url !== null) {
             $element->setAttribute('url', $this->_url);
         }
-        if ($this->_width != null) {
+        if ($this->_width !== null) {
             $element->setAttribute('width', $this->_width);
         }
-        if ($this->_height != null) {
+        if ($this->_height !== null) {
             $element->setAttribute('height', $this->_height);
         }
-        if ($this->_time != null) {
+        if ($this->_time !== null) {
             $element->setAttribute('time', $this->_time);
         }
         return $element;
@@ -109,11 +110,11 @@ class Zend_Gdata_Media_Extension_MediaThumbnail extends Zend_Gdata_Extension
 
     /**
      * Given a DOMNode representing an attribute, tries to map the data into
-     * instance members.  If no mapping is defined, the name and value are 
+     * instance members.  If no mapping is defined, the name and value are
      * stored in an array.
      *
      * @param DOMNode $attribute The DOMNode attribute needed to be handled
-     */ 
+     */
     protected function takeAttributeFromDOM($attribute)
     {
         switch ($attribute->localName) {
@@ -189,7 +190,7 @@ class Zend_Gdata_Media_Extension_MediaThumbnail extends Zend_Gdata_Extension
     }
 
     /**
-     * @return string 
+     * @return string
      */
     public function getTime()
     {
