@@ -15,8 +15,10 @@
  *
  * @category     Zend
  * @package      Zend_Gdata
- * @copyright    Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage   Spreadsheets
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 /**
@@ -34,7 +36,8 @@ require_once 'Zend/Gdata/Spreadsheets/Extension/Cell.php';
  *
  * @category     Zend
  * @package      Zend_Gdata
- * @copyright    Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage   Spreadsheets
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Gdata_Spreadsheets_CellEntry extends Zend_Gdata_Entry
@@ -50,15 +53,13 @@ class Zend_Gdata_Spreadsheets_CellEntry extends Zend_Gdata_Entry
      */
     public function __construct($element = null)
     {
-        foreach (Zend_Gdata_Spreadsheets::$namespaces as $nsPrefix => $nsUri) {
-            $this->registerNamespace($nsPrefix, $nsUri);
-        }
+        $this->registerAllNamespaces(Zend_Gdata_Spreadsheets::$namespaces);
         parent::__construct($element);
     }
 
-    public function getDOM($doc = null)
+    public function getDOM($doc = null, $majorVersion = 1, $minorVersion = null)
     {
-        $element = parent::getDOM($doc);
+        $element = parent::getDOM($doc, $majorVersion, $minorVersion);
         if ($this->_cell != null) {
             $element->appendChild($this->_cell->getDOM($element->ownerDocument));
         }
@@ -91,7 +92,8 @@ class Zend_Gdata_Spreadsheets_CellEntry extends Zend_Gdata_Entry
 
     /**
      * Sets the Cell element of this Cell Entry.
-     * @param $cell Zend_Gdata_Spreadsheets_Extension_Cell $cell
+     * @param Zend_Gdata_Spreadsheets_Extension_Cell $cell
+		 * @return Zend_Gdata_Spreadsheets_CellEntry
      */
     public function setCell($cell)
     {
