@@ -16,9 +16,9 @@
  * @category   Zend
  * @package    Zend_Gdata
  * @subpackage YouTube
- * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: VideoQuery.php 24593 2012-01-05 20:35:02Z matthew $
+ * @version    $Id$
  */
 
 /**
@@ -39,7 +39,7 @@ require_once('Zend/Gdata/Query.php');
  * @category   Zend
  * @package    Zend_Gdata
  * @subpackage YouTube
- * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Gdata_YouTube_VideoQuery extends Zend_Gdata_Query
@@ -92,7 +92,7 @@ class Zend_Gdata_YouTube_VideoQuery extends Zend_Gdata_Query
                     'Video ID must be set for feed of type: ' . $feedType);
             } else {
                 $this->_url = Zend_Gdata_YouTube::VIDEO_URI . '/' . $videoId .
-                    'responses';
+                    '/responses';
             }
             break;
         case 'comments':
@@ -102,7 +102,7 @@ class Zend_Gdata_YouTube_VideoQuery extends Zend_Gdata_Query
                     'Video ID must be set for feed of type: ' . $feedType);
             } else {
                 $this->_url = Zend_Gdata_YouTube::VIDEO_URI . '/' .
-                    $videoId . 'comments';
+                    $videoId . '/comments';
                 if ($entry !== null) {
                     $this->_url .= '/' . $entry;
                 }
