@@ -15,19 +15,24 @@
  * @category   Zend
  * @package    Zend_Http
  * @subpackage Client_Adapter_Exception
- * @version    $Id: Exception.php 2805 2007-01-16 01:38:10Z bkarwin $
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @version    $Id$
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
+/**
+ * @see Zend_Http_Client_Exception
+ */
 require_once 'Zend/Http/Client/Exception.php';
 
 /**
  * @category   Zend
  * @package    Zend_Http
  * @subpackage Client_Adapter
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Http_Client_Adapter_Exception extends Zend_Http_Client_Exception
-{}
+{
+    const READ_TIMEOUT = 1000;
+}
