@@ -14,23 +14,17 @@
  *
  * @category   Zend
  * @package    Zend_Json
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
-
-/**
- * Zend_Json_Exception
- */
-require_once 'Zend/Json/Exception.php';
-
-
 /**
  * Encode PHP constructs to JSON
  *
  * @category   Zend
  * @package    Zend_Json
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Json_Encoder
@@ -41,10 +35,10 @@ class Zend_Json_Encoder
      * @var boolean
      */
     protected $_cycleCheck;
-    
+
     /**
      * Additional options used during encoding
-     * 
+     *
      * @var array
      */
     protected $_options = array();
@@ -80,7 +74,6 @@ class Zend_Json_Encoder
     public static function encode($value, $cycleCheck = false, $options = array())
     {
         $encoder = new self(($cycleCheck) ? true : false, $options);
-
         return $encoder->_encodeValue($value);
     }
 
@@ -91,7 +84,7 @@ class Zend_Json_Encoder
      *    - arrays (returns from {@link _encodeArray()})
      *    - basic datums (e.g. numbers or strings) (returns from {@link _encodeDatum()})
      *
-     * @param $value mixed The value to be encoded
+     * @param mixed $value The value to be encoded
      * @return string Encoded value
      */
     protected function _encodeValue(&$value)
@@ -114,7 +107,7 @@ class Zend_Json_Encoder
      * that contains the name of the class of $value. This is used to decode
      * the object on the client into a specific class.
      *
-     * @param $value object
+     * @param object $value
      * @return string
      * @throws Zend_Json_Exception If recursive checks are enabled and the object has been serialized previously
      */
@@ -122,13 +115,14 @@ class Zend_Json_Encoder
     {
         if ($this->_cycleCheck) {
             if ($this->_wasVisited($value)) {
-                
+
                 if (isset($this->_options['silenceCyclicalExceptions'])
                     && $this->_options['silenceCyclicalExceptions']===true) {
-                    
+
                     return '"* RECURSION (' . get_class($value) . ') *"';
-                    
+
                 } else {
+                    require_once 'Zend/Json/Exception.php';
                     throw new Zend_Json_Exception(
                         'Cycles not supported in JSON encoding, cycle introduced by '
                         . 'class "' . get_class($value) . '"'
@@ -140,16 +134,28 @@ class Zend_Json_Encoder
         }
 
         $props = '';
-        foreach (get_object_vars($value) as $name => $propValue) {
-            if (isset($propValue)) {
-                $props .= ','
-                        . $this->_encodeValue($name)
-                        . ':'
-                        . $this->_encodeValue($propValue);
+        if (method_exists($value, 'toJson')) {
+            $props =',' . preg_replace("/^\{(.*)\}$/","\\1",$value->toJson());
+        } else {
+            if ($value instanceof IteratorAggregate) {
+                $propCollection = $value->getIterator();
+            } elseif ($value instanceof Iterator) {
+                $propCollection = $value;
+            } else {
+                $propCollection = get_object_vars($value);
+            }
+
+            foreach ($propCollection as $name => $propValue) {
+                if (isset($propValue)) {
+                    $props .= ','
+                            . $this->_encodeString($name)
+                            . ':'
+                            . $this->_encodeValue($propValue);
+                }
             }
         }
-
-        return '{"__className":"' . get_class($value) . '"'
+        $className = get_class($value);
+        return '{"__className":' . $this->_encodeString($className)
                 . $props . '}';
     }
 
@@ -180,7 +186,7 @@ class Zend_Json_Encoder
      * the last index is (count($array) -1); any deviation from that is
      * considered an associative array, and will be encoded as such.
      *
-     * @param $array array
+     * @param array& $array
      * @return string
      */
     protected function _encodeArray(&$array)
@@ -220,7 +226,7 @@ class Zend_Json_Encoder
      * If value type is not a string, number, boolean, or null, the string
      * 'null' is returned.
      *
-     * @param $value mixed
+     * @param mixed& $value
      * @return string
      */
     protected function _encodeDatum(&$value)
@@ -228,7 +234,8 @@ class Zend_Json_Encoder
         $result = 'null';
 
         if (is_int($value) || is_float($value)) {
-            $result = (string)$value;
+            $result = (string) $value;
+            $result = str_replace(",", ".", $result);
         } elseif (is_string($value)) {
             $result = $this->_encodeString($value);
         } elseif (is_bool($value)) {
@@ -242,21 +249,22 @@ class Zend_Json_Encoder
     /**
      * JSON encode a string value by escaping characters as necessary
      *
-     * @param $value string
+     * @param string& $value
      * @return string
      */
     protected function _encodeString(&$string)
     {
         // Escape these characters with a backslash:
         // " \ / \n \r \t \b \f
-        $search  = array('\\', "\n", "\t", "\r", "\b", "\f", '"');
-        $replace = array('\\\\', '\\n', '\\t', '\\r', '\\b', '\\f', '\"');
+        $search  = array('\\', "\n", "\t", "\r", "\b", "\f", '"', '/');
+        $replace = array('\\\\', '\\n', '\\t', '\\r', '\\b', '\\f', '\"', '\\/');
         $string  = str_replace($search, $replace, $string);
 
         // Escape certain ASCII characters:
         // 0x08 => \b
         // 0x0c => \f
         $string = str_replace(array(chr(0x08), chr(0x0C)), array('\b', '\f'), $string);
+        $string = self::encodeUnicodeString($string);
 
         return '"' . $string . '"';
     }
@@ -266,7 +274,7 @@ class Zend_Json_Encoder
      * Encode the constants associated with the ReflectionClass
      * parameter. The encoding format is based on the class2 format
      *
-     * @param $cls ReflectionClass
+     * @param ReflectionClass $cls
      * @return string Encoded constant block in class2 format
      */
     private static function _encodeConstants(ReflectionClass $cls)
@@ -291,7 +299,7 @@ class Zend_Json_Encoder
      * Encode the public methods of the ReflectionClass in the
      * class2 format
      *
-     * @param $cls ReflectionClass
+     * @param ReflectionClass $cls
      * @return string Encoded method fragment
      *
      */
@@ -355,7 +363,7 @@ class Zend_Json_Encoder
      * Encode the public properties of the ReflectionClass in the class2
      * format.
      *
-     * @param $cls ReflectionClass
+     * @param ReflectionClass $cls
      * @return string Encode properties list
      *
      */
@@ -387,10 +395,10 @@ class Zend_Json_Encoder
      * NOTE: Currently only public methods and variables are proxied onto
      * the client machine
      *
-     * @param $className string The name of the class, the class must be
-     * instantiable using a null constructor
-     * @param $package string Optional package name appended to JavaScript
-     * proxy class name
+     * @param string $className The name of the class, the class must be
+     *  instantiable using a null constructor
+     * @param string $package Optional package name appended to JavaScript
+     *  proxy class name
      * @return string The class2 (JavaScript) encoding of the class
      * @throws Zend_Json_Exception
      */
@@ -398,6 +406,7 @@ class Zend_Json_Encoder
     {
         $cls = new ReflectionClass($className);
         if (! $cls->isInstantiable()) {
+            require_once 'Zend/Json/Exception.php';
             throw new Zend_Json_Exception("$className must be instantiable");
         }
 
@@ -427,5 +436,143 @@ class Zend_Json_Encoder
         return $result;
     }
 
+    /**
+     * Encode Unicode Characters to \u0000 ASCII syntax.
+     *
+     * This algorithm was originally developed for the
+     * Solar Framework by Paul M. Jones
+     *
+     * @link   http://solarphp.com/
+     * @link   http://svn.solarphp.com/core/trunk/Solar/Json.php
+     * @param  string $value
+     * @return string
+     */
+    public static function encodeUnicodeString($value)
+    {
+        $strlen_var = strlen($value);
+        $ascii = "";
+
+        /**
+         * Iterate over every character in the string,
+         * escaping with a slash or encoding to UTF-8 where necessary
+         */
+        for($i = 0; $i < $strlen_var; $i++) {
+            $ord_var_c = ord($value[$i]);
+
+            switch (true) {
+                case (($ord_var_c >= 0x20) && ($ord_var_c <= 0x7F)):
+                    // characters U-00000000 - U-0000007F (same as ASCII)
+                    $ascii .= $value[$i];
+                    break;
+
+                case (($ord_var_c & 0xE0) == 0xC0):
+                    // characters U-00000080 - U-000007FF, mask 110XXXXX
+                    // see http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                    $char = pack('C*', $ord_var_c, ord($value[$i + 1]));
+                    $i += 1;
+                    $utf16 = self::_utf82utf16($char);
+                    $ascii .= sprintf('\u%04s', bin2hex($utf16));
+                    break;
+
+                case (($ord_var_c & 0xF0) == 0xE0):
+                    // characters U-00000800 - U-0000FFFF, mask 1110XXXX
+                    // see http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                    $char = pack('C*', $ord_var_c,
+                                 ord($value[$i + 1]),
+                                 ord($value[$i + 2]));
+                    $i += 2;
+                    $utf16 = self::_utf82utf16($char);
+                    $ascii .= sprintf('\u%04s', bin2hex($utf16));
+                    break;
+
+                case (($ord_var_c & 0xF8) == 0xF0):
+                    // characters U-00010000 - U-001FFFFF, mask 11110XXX
+                    // see http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                    $char = pack('C*', $ord_var_c,
+                                 ord($value[$i + 1]),
+                                 ord($value[$i + 2]),
+                                 ord($value[$i + 3]));
+                    $i += 3;
+                    $utf16 = self::_utf82utf16($char);
+                    $ascii .= sprintf('\u%04s', bin2hex($utf16));
+                    break;
+
+                case (($ord_var_c & 0xFC) == 0xF8):
+                    // characters U-00200000 - U-03FFFFFF, mask 111110XX
+                    // see http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                    $char = pack('C*', $ord_var_c,
+                                 ord($value[$i + 1]),
+                                 ord($value[$i + 2]),
+                                 ord($value[$i + 3]),
+                                 ord($value[$i + 4]));
+                    $i += 4;
+                    $utf16 = self::_utf82utf16($char);
+                    $ascii .= sprintf('\u%04s', bin2hex($utf16));
+                    break;
+
+                case (($ord_var_c & 0xFE) == 0xFC):
+                    // characters U-04000000 - U-7FFFFFFF, mask 1111110X
+                    // see http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                    $char = pack('C*', $ord_var_c,
+                                 ord($value[$i + 1]),
+                                 ord($value[$i + 2]),
+                                 ord($value[$i + 3]),
+                                 ord($value[$i + 4]),
+                                 ord($value[$i + 5]));
+                    $i += 5;
+                    $utf16 = self::_utf82utf16($char);
+                    $ascii .= sprintf('\u%04s', bin2hex($utf16));
+                    break;
+            }
+        }
+
+        return $ascii;
+     }
+
+    /**
+     * Convert a string from one UTF-8 char to one UTF-16 char.
+     *
+     * Normally should be handled by mb_convert_encoding, but
+     * provides a slower PHP-only method for installations
+     * that lack the multibye string extension.
+     *
+     * This method is from the Solar Framework by Paul M. Jones
+     *
+     * @link   http://solarphp.com
+     * @param string $utf8 UTF-8 character
+     * @return string UTF-16 character
+     */
+    protected static function _utf82utf16($utf8)
+    {
+        // Check for mb extension otherwise do by hand.
+        if( function_exists('mb_convert_encoding') ) {
+            return mb_convert_encoding($utf8, 'UTF-16', 'UTF-8');
+        }
+
+        switch (strlen($utf8)) {
+            case 1:
+                // this case should never be reached, because we are in ASCII range
+                // see: http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                return $utf8;
+
+            case 2:
+                // return a UTF-16 character from a 2-byte UTF-8 char
+                // see: http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                return chr(0x07 & (ord($utf8{0}) >> 2))
+                     . chr((0xC0 & (ord($utf8{0}) << 6))
+                         | (0x3F & ord($utf8{1})));
+
+            case 3:
+                // return a UTF-16 character from a 3-byte UTF-8 char
+                // see: http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+                return chr((0xF0 & (ord($utf8{0}) << 4))
+                         | (0x0F & (ord($utf8{1}) >> 2)))
+                     . chr((0xC0 & (ord($utf8{1}) << 6))
+                         | (0x7F & ord($utf8{2})));
+        }
+
+        // ignoring UTF-32 for now, sorry
+        return '';
+    }
 }
 
