@@ -15,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Ldap
  * @subpackage Node
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Node.php 17829 2009-08-26 15:07:10Z sgehrig $
+ * @version    $Id$
  */
 
 /**
@@ -35,7 +35,7 @@ require_once 'Zend/Ldap/Node/Abstract.php';
  * @category   Zend
  * @package    Zend_Ldap
  * @subpackage Node
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, RecursiveIterator
@@ -99,7 +99,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
     protected function __construct(Zend_Ldap_Dn $dn, array $data, $fromDataSource, Zend_Ldap $ldap = null)
     {
         parent::__construct($dn, $data, $fromDataSource);
-        if (!is_null($ldap)) $this->attachLdap($ldap);
+        if ($ldap !== null) $this->attachLdap($ldap);
         else $this->detachLdap();
     }
 
@@ -136,7 +136,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
      */
     public function getLdap()
     {
-        if (is_null($this->_ldap)) {
+        if ($this->_ldap === null) {
             /**
              * @see Zend_Ldap_Exception
              */
@@ -171,6 +171,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
             $this->_ldap = $ldap;
             if (is_array($this->_children)) {
                 foreach ($this->_children as $child) {
+                    /* @var Zend_Ldap_Node $child */
                     $child->attachLdap($ldap);
                 }
             }
@@ -190,6 +191,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
         $this->_ldap = null;
         if (is_array($this->_children)) {
             foreach ($this->_children as $child) {
+                /* @var Zend_Ldap_Node $child */
                 $child->detachLdap();
             }
         }
@@ -205,7 +207,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
      */
     public function isAttached()
     {
-        return (!is_null($this->_ldap));
+        return ($this->_ldap !== null);
     }
 
     /**
@@ -258,7 +260,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
      *
      * @param  string|array|Zend_Ldap_Dn $dn
      * @param  Zend_Ldap                 $ldap
-     * @return Zend_Ldap_Node
+     * @return Zend_Ldap_Node|null
      * @throws Zend_Ldap_Exception
      */
     public static function fromLdap($dn, Zend_Ldap $ldap)
@@ -275,6 +277,9 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
             throw new Zend_Ldap_Exception(null, '$dn is of a wrong data type.');
         }
         $data = $ldap->getEntry($dn, array('*', '+'), true);
+        if ($data === null) {
+            return null;
+        }
         $entry = new self($dn, $data, true, $ldap);
         return $entry;
     }
@@ -316,12 +321,17 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
     /**
      * Ensures that teh RDN attributes are correctly set.
      *
+     * @param  boolean    $overwrite    True to overwrite the RDN attributes
      * @return void
      */
-    protected function _ensureRdnAttributeValues()
+    protected function _ensureRdnAttributeValues($overwrite = false)
     {
         foreach ($this->getRdnArray() as $key => $value) {
-            Zend_Ldap_Attribute::setAttribute($this->_currentData, $key, $value, false);
+            if (!array_key_exists($key, $this->_currentData) || $overwrite) {
+                Zend_Ldap_Attribute::setAttribute($this->_currentData, $key, $value, false);
+            } else if (!in_array($value, $this->_currentData[$key])) {
+                Zend_Ldap_Attribute::setAttribute($this->_currentData, $key, $value, true);
+            }
         }
     }
 
@@ -425,20 +435,25 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
 
         if ($this->willBeDeleted()) {
             if ($ldap->exists($this->_dn)) {
+                $this->_preDelete();
                 $ldap->delete($this->_dn);
+                $this->_postDelete();
             }
             return $this;
         }
 
         if ($this->isNew()) {
+            $this->_preAdd();
             $data = $this->getData();
             $ldap->add($this->_getDn(), $data);
             $this->_loadData($data, true);
+            $this->_postAdd();
             return $this;
         }
 
         $changedData = $this->getChangedData();
         if ($this->willBeMoved()) {
+            $this->_preRename();
             $recursive = $this->hasChildren();
             $ldap->rename($this->_dn, $this->_newDn, $recursive, false);
             foreach ($this->_newDn->getRdn() as $key => $value) {
@@ -448,9 +463,12 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
             }
             $this->_dn = $this->_newDn;
             $this->_newDn = null;
+            $this->_postRename();
         }
         if (count($changedData) > 0) {
+            $this->_preUpdate();
             $ldap->update($this->_getDn(), $changedData);
+            $this->_postUpdate();
         }
         $this->_originalData = $this->_currentData;
         return $this;
@@ -498,7 +516,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
         } else {
             $this->_newDn = Zend_Ldap_Dn::factory($newDn);
         }
-        $this->_ensureRdnAttributeValues();
+        $this->_ensureRdnAttributeValues(true);
         return $this;
     }
 
@@ -1016,6 +1034,7 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
             if ($this->isAttached()) {
                 $children = $this->searchChildren('(objectClass=*)', null);
                 foreach ($children as $child) {
+                    /* @var Zend_Ldap_Node $child */
                     $this->_children[$child->getRdnString(Zend_Ldap_Dn::ATTR_CASEFOLD_LOWER)] = $child;
                 }
             }
@@ -1095,4 +1114,72 @@ class Zend_Ldap_Node extends Zend_Ldap_Node_Abstract implements Iterator, Recurs
     {
         return $this->_iteratorRewind;
     }
-}
\ No newline at end of file
+
+    ####################################################
+    # Empty method bodies for overriding in subclasses #
+    ####################################################
+
+    /**
+     * Allows pre-delete logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _preDelete() { }
+
+    /**
+     * Allows post-delete logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _postDelete() { }
+
+    /**
+     * Allows pre-add logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _preAdd() { }
+
+    /**
+     * Allows post-add logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _postAdd() { }
+
+    /**
+     * Allows pre-rename logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _preRename() { }
+
+    /**
+     * Allows post-rename logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _postRename() { }
+
+    /**
+     * Allows pre-update logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _preUpdate() { }
+
+    /**
+     * Allows post-update logic to be applied to node.
+     * Subclasses may override this method.
+     *
+     * @return void
+     */
+    protected function _postUpdate() { }
+}
