@@ -15,26 +15,23 @@
  * @category   Zend
  * @package    Zend_Log
  * @subpackage Formatter
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Simple.php 10726 2008-08-06 16:38:09Z cadorn $
+ * @version    $Id$
  */
 
-/** Zend_Log_Formatter_Interface */
-require_once 'Zend/Log/Formatter/Interface.php';
-
-/** Zend_Log_Exception */
-require_once 'Zend/Log/Exception.php';
+/** Zend_Log_Formatter_Abstract */
+require_once 'Zend/Log/Formatter/Abstract.php';
 
 /**
  * @category   Zend
  * @package    Zend_Log
  * @subpackage Formatter
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Simple.php 10726 2008-08-06 16:38:09Z cadorn $
+ * @version    $Id$
  */
-class Zend_Log_Formatter_Simple implements Zend_Log_Formatter_Interface
+class Zend_Log_Formatter_Simple extends Zend_Log_Formatter_Abstract
 {
     /**
      * @var string
@@ -47,6 +44,7 @@ class Zend_Log_Formatter_Simple implements Zend_Log_Formatter_Interface
      * Class constructor
      *
      * @param  null|string  $format  Format specifier for log messages
+     * @return void
      * @throws Zend_Log_Exception
      */
     public function __construct($format = null)
@@ -55,13 +53,36 @@ class Zend_Log_Formatter_Simple implements Zend_Log_Formatter_Interface
             $format = self::DEFAULT_FORMAT . PHP_EOL;
         }
 
-        if (! is_string($format)) {
+        if (!is_string($format)) {
+            require_once 'Zend/Log/Exception.php';
             throw new Zend_Log_Exception('Format must be a string');
         }
 
         $this->_format = $format;
     }
 
+    /**
+	 * Factory for Zend_Log_Formatter_Simple classe
+	 *
+	 * @param array|Zend_Config $options
+	 * @return Zend_Log_Formatter_Simple
+     */
+    public static function factory($options)
+    {
+        $format = null;
+        if (null !== $options) {
+            if ($options instanceof Zend_Config) {
+                $options = $options->toArray();
+            }
+
+            if (array_key_exists('format', $options)) {
+                $format = $options['format'];
+            }
+        }
+
+        return new self($format);
+    }
+
     /**
      * Formats data into a single line to be written by the writer.
      *
@@ -71,17 +92,17 @@ class Zend_Log_Formatter_Simple implements Zend_Log_Formatter_Interface
     public function format($event)
     {
         $output = $this->_format;
+
         foreach ($event as $name => $value) {
-
             if ((is_object($value) && !method_exists($value,'__toString'))
-                || is_array($value)) {
-
-                $value = gettype($value);  
+                || is_array($value)
+            ) {
+                $value = gettype($value);
             }
 
             $output = str_replace("%$name%", $value, $output);
         }
+
         return $output;
     }
-
 }
