@@ -12,10 +12,12 @@
  * obtain it through the world-wide-web, please send an email
  * to license@zend.com so we can send you a copy immediately.
  *
+ * @category   Zend
  * @package    Zend_Pdf
  * @subpackage Fonts
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 /** Zend_Pdf_Cmap */
@@ -31,7 +33,7 @@ require_once 'Zend/Pdf/Cmap.php';
  *
  * @package    Zend_Pdf
  * @subpackage Fonts
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Pdf_Cmap_TrimmedTable extends Zend_Pdf_Cmap
@@ -132,9 +134,9 @@ class Zend_Pdf_Cmap_TrimmedTable extends Zend_Pdf_Cmap
     /**
      * Returns an array containing the glyphs numbers that have entries in this character map.
      * Keys are Unicode character codes (integers)
-     * 
+     *
      * This functionality is partially covered by glyphNumbersForCharacters(getCoveredCharacters())
-     * call, but this method do it in more effective way (prepare complete list instead of searching 
+     * call, but this method do it in more effective way (prepare complete list instead of searching
      * glyph for each character code).
      *
      * @internal
@@ -142,7 +144,7 @@ class Zend_Pdf_Cmap_TrimmedTable extends Zend_Pdf_Cmap
      */
     public function getCoveredCharactersGlyphs()
     {
-    	$glyphNumbers = array();
+        $glyphNumbers = array();
         for ($code = $this->_startCode; $code <= $this->_endCode; $code++) {
             $glyphNumbers[$code] = $this->_glyphIndexArray[$code - $this->_startCode];
         }
@@ -168,6 +170,7 @@ class Zend_Pdf_Cmap_TrimmedTable extends Zend_Pdf_Cmap
          */
         $actualLength = strlen($cmapData);
         if ($actualLength < 9) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception('Insufficient table data',
                                          Zend_Pdf_Exception::CMAP_TABLE_DATA_TOO_SMALL);
         }
@@ -176,12 +179,14 @@ class Zend_Pdf_Cmap_TrimmedTable extends Zend_Pdf_Cmap
          */
         $type = $this->_extractUInt2($cmapData, 0);
         if ($type != Zend_Pdf_Cmap::TYPE_TRIMMED_TABLE) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception('Wrong cmap table type',
                                          Zend_Pdf_Exception::CMAP_WRONG_TABLE_TYPE);
         }
 
         $length = $this->_extractUInt2($cmapData, 2);
         if ($length != $actualLength) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception("Table length ($length) does not match actual length ($actualLength)",
                                          Zend_Pdf_Exception::CMAP_WRONG_TABLE_LENGTH);
         }
@@ -201,6 +206,7 @@ class Zend_Pdf_Cmap_TrimmedTable extends Zend_Pdf_Cmap
         $entryCount = $this->_extractUInt2($cmapData, 8);
         $expectedCount = ($length - 10) >> 1;
         if ($entryCount != $expectedCount) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception("Entry count is wrong; expected: $expectedCount; actual: $entryCount",
                                          Zend_Pdf_Exception::CMAP_WRONG_ENTRY_COUNT);
         }
@@ -216,6 +222,7 @@ class Zend_Pdf_Cmap_TrimmedTable extends Zend_Pdf_Cmap
          * of the table.
          */
         if ($offset != $length) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception("Ending offset ($offset) does not match length ($length)",
                                          Zend_Pdf_Exception::CMAP_FINAL_OFFSET_NOT_LENGTH);
         }
