@@ -14,23 +14,14 @@
  *
  * @category   Zend
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
-
-/** Zend_Pdf_Exception */
-require_once 'Zend/Pdf/Exception.php';
-
 /** Zend_Pdf_Color */
 require_once 'Zend/Pdf/Color.php';
 
-/** Zend_Pdf_Color_Rgb */
-require_once 'Zend/Pdf/Color/Rgb.php';
-
-/** Zend_Pdf_GrayScale */
-require_once 'Zend/Pdf/Color/GrayScale.php';
-
 
 /**
  * HTML color implementation
@@ -40,7 +31,7 @@ require_once 'Zend/Pdf/Color/GrayScale.php';
  *
  * @category   Zend
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Pdf_Color_Html extends Zend_Pdf_Color
@@ -78,6 +69,16 @@ class Zend_Pdf_Color_Html extends Zend_Pdf_Color
         return $this->_color->instructions($stroking);
     }
 
+    /**
+     * Get color components (color space dependent)
+     *
+     * @return array
+     */
+    public function getComponents()
+    {
+        return $this->_color->getComponents();
+    }
+
     /**
      * Creates a Zend_Pdf_Color object from the HTML representation.
      *
@@ -93,8 +94,10 @@ class Zend_Pdf_Color_Html extends Zend_Pdf_Color
             $g = round((hexdec($matches[2]) / 255), 3);
             $b = round((hexdec($matches[3]) / 255), 3);
             if (($r == $g) && ($g == $b)) {
+                require_once 'Zend/Pdf/Color/GrayScale.php';
                 return new Zend_Pdf_Color_GrayScale($r);
             } else {
+                require_once 'Zend/Pdf/Color/Rgb.php';
                 return new Zend_Pdf_Color_Rgb($r, $g, $b);
             }
         } else {
@@ -395,13 +398,15 @@ class Zend_Pdf_Color_Html extends Zend_Pdf_Color
                 $r = 0.604; $g = 0.804; $b = 0.196; break;
 
             default:
+                require_once 'Zend/Pdf/Exception.php';
                 throw new Zend_Pdf_Exception('Unknown color name: ' . $color);
         }
         if (($r == $g) && ($g == $b)) {
+            require_once 'Zend/Pdf/Color/GrayScale.php';
             return new Zend_Pdf_Color_GrayScale($r);
         } else {
+            require_once 'Zend/Pdf/Color/Rgb.php';
             return new Zend_Pdf_Color_Rgb($r, $g, $b);
         }
     }
 }
-
