@@ -12,23 +12,21 @@
  * obtain it through the world-wide-web, please send an email
  * to license@zend.com so we can send you a copy immediately.
  *
+ * @category   Zend
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 
-/** Zend_Pdf */
-require_once 'Zend/Pdf.php';
-
-
 /**
  * Zend_Pdf_ImageFactory
  *
  * Helps manage the diverse set of supported image file types.
  *
  * @package    Zend_Pdf
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  * @todo       Use Zend_Mime not file extension for type determination.
  */
@@ -36,6 +34,7 @@ class Zend_Pdf_Resource_ImageFactory
 {
     public static function factory($filename) {
         if(!is_file($filename)) {
+            require_once 'Zend/Pdf/Exception.php';
             throw new Zend_Pdf_Exception("Cannot create image resource. File not found.");
         }
         $extension = pathinfo($filename, PATHINFO_EXTENSION);
@@ -47,9 +46,11 @@ class Zend_Pdf_Resource_ImageFactory
             case 'tif':
                 //Fall through to next case;
             case 'tiff':
+                require_once 'Zend/Pdf/Resource/Image/Tiff.php';
                 return new Zend_Pdf_Resource_Image_Tiff($filename);
                 break;
             case 'png':
+                require_once 'Zend/Pdf/Resource/Image/Png.php';
                 return new Zend_Pdf_Resource_Image_Png($filename);
                 break;
             case 'jpg':
@@ -57,9 +58,11 @@ class Zend_Pdf_Resource_ImageFactory
             case 'jpe':
                 //Fall through to next case;
             case 'jpeg':
+                require_once 'Zend/Pdf/Resource/Image/Jpeg.php';
                 return new Zend_Pdf_Resource_Image_Jpeg($filename);
                 break;
             default:
+                require_once 'Zend/Pdf/Exception.php';
                 throw new Zend_Pdf_Exception("Cannot create image resource. File extension not known or unsupported type.");
                 break;
         }
