@@ -15,46 +15,11 @@
  * @category   Zend
  * @package    Zend_Service_WindowsAzure
  * @subpackage Storage
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  * @version    $Id$
  */
 
-/**
- * @see Zend_Service_WindowsAzure_Credentials_CredentialsAbstract
- */
-require_once 'Zend/Service/WindowsAzure/Credentials/CredentialsAbstract.php';
-
-/**
- * @see Zend_Service_WindowsAzure_Credentials_SharedKey
- */
-require_once 'Zend/Service/WindowsAzure/Credentials/SharedKey.php';
-
-/**
- * @see Zend_Service_WindowsAzure_Credentials_SharedKeyLite
- */
-require_once 'Zend/Service/WindowsAzure/Credentials/SharedKeyLite.php';
-
-/**
- * @see Zend_Service_WindowsAzure_RetryPolicy_RetryPolicyAbstract
- */
-require_once 'Zend/Service/WindowsAzure/RetryPolicy/RetryPolicyAbstract.php';
-
-/**
- * @see Zend_Http_Client
- */
-require_once 'Zend/Http/Client.php';
-
-/**
- * @see Zend_Http_Response
- */
-require_once 'Zend/Http/Response.php';
-
-/**
- * @see Zend_Service_WindowsAzure_Storage
- */
-require_once 'Zend/Service/WindowsAzure/Storage.php';
-
 /**
  * @see Zend_Service_WindowsAzure_Storage_BatchStorageAbstract
  */
@@ -66,9 +31,9 @@ require_once 'Zend/Service/WindowsAzure/Storage/BatchStorageAbstract.php';
 require_once 'Zend/Service/WindowsAzure/Storage/TableInstance.php';
 
 /**
- * @see Zend_Service_WindowsAzure_Storage_TableEntity
+ * @see Zend_Service_WindowsAzure_Storage_TableEntityQuery
  */
-require_once 'Zend/Service/WindowsAzure/Storage/TableEntity.php';
+require_once 'Zend/Service/WindowsAzure/Storage/TableEntityQuery.php';
 
 /**
  * @see Zend_Service_WindowsAzure_Storage_DynamicTableEntity
@@ -76,26 +41,47 @@ require_once 'Zend/Service/WindowsAzure/Storage/TableEntity.php';
 require_once 'Zend/Service/WindowsAzure/Storage/DynamicTableEntity.php';
 
 /**
- * @see Zend_Service_WindowsAzure_Storage_TableEntityQuery
+ * @see Zend_Service_WindowsAzure_Credentials_SharedKeyLite 
  */
-require_once 'Zend/Service/WindowsAzure/Storage/TableEntityQuery.php';
-
-/**
- * @see Zend_Service_WindowsAzure_Exception
- */
-require_once 'Zend/Service/WindowsAzure/Exception.php';
-
+require_once 'Zend/Service/WindowsAzure/Credentials/SharedKeyLite.php';
 
 /**
  * @category   Zend
  * @package    Zend_Service_WindowsAzure
  * @subpackage Storage
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Service_WindowsAzure_Storage_Table
     extends Zend_Service_WindowsAzure_Storage_BatchStorageAbstract
 {
+	/**
+	 * Throw Zend_Service_WindowsAzure_Exception when a property is not specified in Windows Azure?
+	 * Defaults to true, making behaviour similar to Windows Azure StorageClient in .NET.
+	 * 
+	 * @var boolean
+	 */
+	protected $_throwExceptionOnMissingData = true;
+	
+	/**
+	 * Throw Zend_Service_WindowsAzure_Exception when a property is not specified in Windows Azure?
+	 * Defaults to true, making behaviour similar to Windows Azure StorageClient in .NET.
+	 * 
+	 * @param boolean $value
+	 */
+	public function setThrowExceptionOnMissingData($value = true)
+	{
+		$this->_throwExceptionOnMissingData = $value;
+	}
+	
+	/**
+	 * Throw Zend_Service_WindowsAzure_Exception when a property is not specified in Windows Azure?
+	 */
+	public function getThrowExceptionOnMissingData()
+	{
+		return $this->_throwExceptionOnMissingData;
+	}
+	
 	/**
 	 * Creates a new Zend_Service_WindowsAzure_Storage_Table instance
 	 *
@@ -125,6 +111,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 	public function tableExists($tableName = '')
 	{
 		if ($tableName === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Table name is not specified.');
 		}
 			
@@ -188,6 +175,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    
 			// More tables?
 		    if (!is_null($response->getHeader('x-ms-continuation-NextTableName'))) {
+				require_once 'Zend/Service/WindowsAzure/Exception.php';
 		        $returnValue = array_merge($returnValue, $this->listTables($response->getHeader('x-ms-continuation-NextTableName')));
 		    }
 
@@ -251,6 +239,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    $tableName = $entry->xpath('.//m:properties/d:TableName');
 		    $tableName = (string)$tableName[0];
 		        
+			
 		    return new Zend_Service_WindowsAzure_Storage_TableInstance(
 		        (string)$entry->id,
 		        $tableName,
@@ -258,7 +247,21 @@ class Zend_Service_WindowsAzure_Storage_Table
 		        (string)$entry->updated
 		    );
 		} else {
-			throw new Zend_Service_WindowsAzure_Exception($this->_getErrorMessage($response, 'Resource could not be accessed.'));
+                    require_once 'Zend/Service/WindowsAzure/Exception.php';
+                    throw new Zend_Service_WindowsAzure_Exception($this->_getErrorMessage($response, 'Resource could not be accessed.'));
+		}
+	}
+	
+	/**
+	 * Create table if it does not exist
+	 *
+	 * @param string $tableName Table name
+	 * @throws Zend_Service_WindowsAzure_Exception
+	 */
+	public function createTableIfNotExists($tableName = '')
+	{
+		if (!$this->tableExists($tableName)) {
+			$this->createTable($tableName);
 		}
 	}
 	
@@ -271,6 +274,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 	public function deleteTable($tableName = '')
 	{
 		if ($tableName === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Table name is not specified.');
 		}
 
@@ -281,6 +285,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		// Perform request
 		$response = $this->_performRequest('Tables(\'' . $tableName . '\')', '', Zend_Http_Client::DELETE, $headers, true, null);
 		if (!$response->isSuccessful()) {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception($this->_getErrorMessage($response, 'Resource could not be accessed.'));
 		}
 	}
@@ -296,9 +301,11 @@ class Zend_Service_WindowsAzure_Storage_Table
 	public function insertEntity($tableName = '', Zend_Service_WindowsAzure_Storage_TableEntity $entity = null)
 	{
 		if ($tableName === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Table name is not specified.');
 		}
 		if (is_null($entity)) {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Entity is not specified.');
 		}
 		                     
@@ -340,7 +347,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    $result = $this->_parseResponse($response);
 		    
 		    $timestamp = $result->xpath('//m:properties/d:Timestamp');
-		    $timestamp = (string)$timestamp[0];
+		    $timestamp = $this->_convertToDateTime( (string)$timestamp[0] );
 
 		    $etag      = $result->attributes('http://schemas.microsoft.com/ado/2007/08/dataservices/metadata');
 		    $etag      = (string)$etag['etag'];
@@ -351,6 +358,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 
 		    return $entity;
 		} else {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception($this->_getErrorMessage($response, 'Resource could not be accessed.'));
 		}
 	}
@@ -366,9 +374,11 @@ class Zend_Service_WindowsAzure_Storage_Table
 	public function deleteEntity($tableName = '', Zend_Service_WindowsAzure_Storage_TableEntity $entity = null, $verifyEtag = false)
 	{
 		if ($tableName === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Table name is not specified.');
 		}
 		if (is_null($entity)) {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Entity is not specified.');
 		}
 		                     
@@ -394,6 +404,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    $response = $this->_performRequest($tableName . '(PartitionKey=\'' . $entity->getPartitionKey() . '\', RowKey=\'' . $entity->getRowKey() . '\')', '', Zend_Http_Client::DELETE, $headers, true, null);
 		}
 		if (!$response->isSuccessful()) {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 		    throw new Zend_Service_WindowsAzure_Exception($this->_getErrorMessage($response, 'Resource could not be accessed.'));
 		}
 	}
@@ -408,18 +419,22 @@ class Zend_Service_WindowsAzure_Storage_Table
 	 * @return Zend_Service_WindowsAzure_Storage_TableEntity
 	 * @throws Zend_Service_WindowsAzure_Exception
 	 */
-	public function retrieveEntityById($tableName = '', $partitionKey = '', $rowKey = '', $entityClass = 'Zend_Service_WindowsAzure_Storage_DynamicTableEntity')
+	public function retrieveEntityById($tableName, $partitionKey, $rowKey, $entityClass = 'Zend_Service_WindowsAzure_Storage_DynamicTableEntity')
 	{
-		if ($tableName === '') {
+		if (is_null($tableName) || $tableName === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Table name is not specified.');
 		}
-		if ($partitionKey === '') {
+		if (is_null($partitionKey) || $partitionKey === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Partition key is not specified.');
 		}
-		if ($rowKey === '') {
+		if (is_null($rowKey) || $rowKey === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Row key is not specified.');
 		}
-		if ($entityClass === '') {
+		if (is_null($entityClass) || $entityClass === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Entity class is not specified.');
 		}
 
@@ -429,6 +444,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		if (strlen($partitionKey . $rowKey) >= 256) {
 		    // Start a batch if possible
 		    if ($this->isInBatch()) {
+				require_once 'Zend/Service/WindowsAzure/Exception.php';
 		        throw new Zend_Service_WindowsAzure_Exception('Entity cannot be retrieved. A transaction is required to retrieve the entity, but another transaction is already active.');
 		    }
 		        
@@ -460,6 +476,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 	 */
 	public function select()
 	{
+            
 	    return new Zend_Service_WindowsAzure_Storage_TableEntityQuery();
 	}
 	
@@ -477,9 +494,11 @@ class Zend_Service_WindowsAzure_Storage_Table
 	public function retrieveEntities($tableName = '', $filter = '', $entityClass = 'Zend_Service_WindowsAzure_Storage_DynamicTableEntity', $nextPartitionKey = null, $nextRowKey = null)
 	{
 		if ($tableName === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Table name is not specified.');
 		}
 		if ($entityClass === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Entity class is not specified.');
 		}
 
@@ -497,7 +516,9 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    // Option 1: $tableName is a string
 		    
 		    // Append parentheses
-		    $tableName .= '()';
+		    if (strpos($tableName, '()') === false) {
+		    	$tableName .= '()';
+		    }
 		    
     	    // Build query
     	    $query = array();
@@ -520,6 +541,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    // Change $tableName
 		    $tableName = $tableName->assembleFrom(true);
 		} else {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 		    throw new Zend_Service_WindowsAzure_Exception('Invalid argument: $tableName');
 		}
 		
@@ -527,9 +549,11 @@ class Zend_Service_WindowsAzure_Storage_Table
 		if (!is_null($nextPartitionKey) && !is_null($nextRowKey)) {
 		    if ($queryString !== '') {
 		        $queryString .= '&';
+		    } else {
+		    	$queryString .= '?';
 		    }
 		        
-		    $queryString .= '&NextPartitionKey=' . rawurlencode($nextPartitionKey) . '&NextRowKey=' . rawurlencode($nextRowKey);
+		    $queryString .= 'NextPartitionKey=' . rawurlencode($nextPartitionKey) . '&NextRowKey=' . rawurlencode($nextRowKey);
 		}
 
 		// Perform request
@@ -580,15 +604,15 @@ class Zend_Service_WindowsAzure_Storage_Table
     		    
     		    // Create entity
     		    $entity = new $entityClass('', '');
-    		    $entity->setAzureValues((array)$properties, true);
+    		    $entity->setAzureValues((array)$properties, $this->_throwExceptionOnMissingData);
     		    
-    		    // If we have a Zend_Service_WindowsAzure_Storage_DynamicTableEntity, make sure all property types are OK
+    		    // If we have a Zend_Service_WindowsAzure_Storage_DynamicTableEntity, make sure all property types are set
     		    if ($entity instanceof Zend_Service_WindowsAzure_Storage_DynamicTableEntity) {
     		        foreach ($properties as $key => $value) {  
     		            $attributes = $value->attributes('http://schemas.microsoft.com/ado/2007/08/dataservices/metadata');
     		            $type = (string)$attributes['type'];
     		            if ($type !== '') {
-    		                $entity->setAzurePropertyType($key, $type);
+    		            	$entity->setAzureProperty($key, (string)$value, $type);
     		            }
     		        }
     		    }
@@ -612,6 +636,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    // Return
 		    return $returnValue;
 		} else {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 		    throw new Zend_Service_WindowsAzure_Exception($this->_getErrorMessage($response, 'Resource could not be accessed.'));
 		}
 	}
@@ -642,6 +667,7 @@ class Zend_Service_WindowsAzure_Storage_Table
 	{
 		$mergeEntity = null;
 		if (is_array($properties) && count($properties) > 0) {
+			
 			// Build a new object
 			$mergeEntity = new Zend_Service_WindowsAzure_Storage_DynamicTableEntity($entity->getPartitionKey(), $entity->getRowKey());
 			
@@ -656,6 +682,9 @@ class Zend_Service_WindowsAzure_Storage_Table
 			$mergeEntity = $entity;
 		}
 		
+		// Ensure entity timestamp matches updated timestamp 
+        $entity->setTimestamp(new DateTime());
+        
 	    return $this->_changeEntity(Zend_Http_Client::MERGE, $tableName, $mergeEntity, $verifyEtag);
 	}
 	
@@ -688,9 +717,11 @@ class Zend_Service_WindowsAzure_Storage_Table
 	protected function _changeEntity($httpVerb = Zend_Http_Client::PUT, $tableName = '', Zend_Service_WindowsAzure_Storage_TableEntity $entity = null, $verifyEtag = false)
 	{
 		if ($tableName === '') {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Table name is not specified.');
 		}
-		if (is_null($entity)) {
+		if (is_null($entity)) {	
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception('Entity is not specified.');
 		}
 		                     
@@ -720,8 +751,11 @@ class Zend_Service_WindowsAzure_Storage_Table
                           </content>
                         </entry>';
 		
+		// Attempt to get timestamp from entity
+        $timestamp = $entity->getTimestamp();
+        
         $requestBody = $this->_fillTemplate($requestBody, array(
-        	'Updated'    => $this->isoDate(),
+        	'Updated'    => $this->_convertToEdmDateTime($timestamp),
             'Properties' => $this->_generateAzureRepresentation($entity)
         ));
 
@@ -745,10 +779,11 @@ class Zend_Service_WindowsAzure_Storage_Table
 		if ($response->isSuccessful()) {
 		    // Update properties
 			$entity->setEtag($response->getHeader('Etag'));
-			$entity->setTimestamp($response->getHeader('Last-modified'));
+			$entity->setTimestamp( $this->_convertToDateTime($response->getHeader('Last-modified')) );
 
 		    return $entity;
 		} else {
+			require_once 'Zend/Service/WindowsAzure/Exception.php';
 			throw new Zend_Service_WindowsAzure_Exception($this->_getErrorMessage($response, 'Resource could not be accessed.'));
 		}
 	}
@@ -803,6 +838,8 @@ class Zend_Service_WindowsAzure_Storage_Table
 		    if (!is_null($azureValue->Value)) {
 		        if (strtolower($azureValue->Type) == 'edm.boolean') {
 		            $value[] = ($azureValue->Value == true ? '1' : '0');
+		        } else if (strtolower($azureValue->Type) == 'edm.datetime') {
+		        	$value[] = $this->_convertToEdmDateTime($azureValue->Value);
 		        } else {
 		            $value[] = htmlspecialchars($azureValue->Value);
 		        }
@@ -853,5 +890,42 @@ class Zend_Service_WindowsAzure_Storage_Table
 			$resourceType,
 			$requiredPermission
 		);
-	}
+	}  
+	  
+    /**
+     * Converts a string to a DateTime object. Returns false on failure.
+     * 
+     * @param string $value The string value to parse
+     * @return DateTime|boolean
+     */
+    protected function _convertToDateTime($value = '') 
+    {
+    	if ($value instanceof DateTime) {
+    		return $value;
+    	}
+    	
+    	try {
+    		if (substr($value, -1) == 'Z') {
+    			$value = substr($value, 0, strlen($value) - 1);
+    		}
+    		return new DateTime($value, new DateTimeZone('UTC'));
+    	}
+    	catch (Exception $ex) {
+    		return false;
+    	}
+    }
+    
+    /**
+     * Converts a DateTime object into an Edm.DaeTime value in UTC timezone,
+     * represented as a string.
+     * 
+     * @param DateTime $value
+     * @return string
+     */
+    protected function _convertToEdmDateTime(DateTime $value) 
+    {
+    	$cloned = clone $value;
+    	$cloned->setTimezone(new DateTimeZone('UTC'));
+    	return str_replace('+0000', 'Z', $cloned->format(DateTime::ISO8601));
+    }
 }
