@@ -14,43 +14,29 @@
  *
  * @category   Zend
  * @package    Zend_Translate
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
- * @version    $Id: Date.php 2498 2006-12-23 22:13:38Z thomas $
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
+ * @version    $Id$
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
-
 /** Zend_Locale */
 require_once 'Zend/Locale.php';
 
 /** Zend_Translate_Adapter */
 require_once 'Zend/Translate/Adapter.php';
 
-
 /**
  * @category   Zend
  * @package    Zend_Translate
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Translate_Adapter_Gettext extends Zend_Translate_Adapter {
     // Internal variables
     private $_bigEndian   = false;
     private $_file        = false;
-
-    /**
-     * Generates the  adapter
-     *
-     * @param  string              $data     Translation data
-     * @param  string|Zend_Locale  $locale   OPTIONAL Locale/Language to set, identical with locale identifier,
-     *                                       see Zend_Locale for more information
-     * @param  array               $options  OPTIONAL Options to set
-     */
-    public function __construct($data, $locale = null, array $options = array())
-    {
-        parent::__construct($data, $locale, $options);
-    }
-
+    private $_adapterInfo = array();
+    private $_data        = array();
 
     /**
      * Read values from the MO file
@@ -66,7 +52,6 @@ class Zend_Translate_Adapter_Gettext extends Zend_Translate_Adapter {
         }
     }
 
-
     /**
      * Load translation data (MO file reader)
      *
@@ -75,22 +60,19 @@ class Zend_Translate_Adapter_Gettext extends Zend_Translate_Adapter {
      *                            see Zend_Locale for more information
      * @param  array   $option    OPTIONAL Options to use
      * @throws Zend_Translation_Exception
+     * @return array
      */
     protected function _loadTranslationData($filename, $locale, array $options = array())
     {
+        $this->_data      = array();
         $this->_bigEndian = false;
-        $options = $options + $this->_options;
-
-        if ($options['clear']  ||  !isset($this->_translate[$locale])) {
-            $this->_translate[$locale] = array();
-        }
-
-        $this->_file = @fopen($filename, 'rb');
+        $this->_file      = @fopen($filename, 'rb');
         if (!$this->_file) {
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception('Error opening translation file \'' . $filename . '\'.');
         }
         if (@filesize($filename) < 10) {
+            @fclose($this->_file);
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception('\'' . $filename . '\' is not a gettext file');
         }
@@ -102,6 +84,7 @@ class Zend_Translate_Adapter_Gettext extends Zend_Translate_Adapter {
         } else if (strtolower(substr(dechex($input[1]), -8)) == "de120495") {
             $this->_bigEndian = true;
         } else {
+            @fclose($this->_file);
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception('\'' . $filename . '\' is not a gettext file');
         }
@@ -130,17 +113,49 @@ class Zend_Translate_Adapter_Gettext extends Zend_Translate_Adapter {
             if ($origtemp[$count * 2 + 1] != 0) {
                 fseek($this->_file, $origtemp[$count * 2 + 2]);
                 $original = @fread($this->_file, $origtemp[$count * 2 + 1]);
+                $original = explode("\0", $original);
             } else {
-                $original = '';
+                $original[0] = '';
             }
 
             if ($transtemp[$count * 2 + 1] != 0) {
                 fseek($this->_file, $transtemp[$count * 2 + 2]);
-                $this->_translate[$locale][$original] = fread($this->_file, $transtemp[$count * 2 + 1]);
+                $translate = fread($this->_file, $transtemp[$count * 2 + 1]);
+                $translate = explode("\0", $translate);
+                if ((count($original) > 1)) {
+                    $this->_data[$locale][$original[0]] = $translate;
+                    array_shift($original);
+                    foreach ($original as $orig) {
+                        $this->_data[$locale][$orig] = '';
+                    }
+                } else {
+                    $this->_data[$locale][$original[0]] = $translate[0];
+                }
             }
         }
+        
+        @fclose($this->_file);
+
+        $this->_data[$locale][''] = trim($this->_data[$locale]['']);
+        if (empty($this->_data[$locale][''])) {
+            $this->_adapterInfo[$filename] = 'No adapter information available';
+        } else {
+            $this->_adapterInfo[$filename] = $this->_data[$locale][''];
+        }
+
+        unset($this->_data[$locale]['']);
+        return $this->_data;
     }
 
+    /**
+     * Returns the adapter informations
+     *
+     * @return array Each loaded adapter information as array value
+     */
+    public function getAdapterInfo()
+    {
+        return $this->_adapterInfo;
+    }
 
     /**
      * Returns the adapter name
