@@ -14,8 +14,8 @@
  *
  * @category   Zend
  * @package    Zend_Translate
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
- * @version    $Id: Tmx.php 16971 2009-07-22 18:05:45Z mikaelkael $
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
+ * @version    $Id$
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -26,37 +26,29 @@ require_once 'Zend/Locale.php';
 /** Zend_Translate_Adapter */
 require_once 'Zend/Translate/Adapter.php';
 
+/** @see Zend_Xml_Security */
+require_once 'Zend/Xml/Security.php';
+
+/** @See Zend_Xml_Exception */
+require_once 'Zend/Xml/Exception.php';
 
 /**
  * @category   Zend
  * @package    Zend_Translate
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
     // Internal variables
     private $_file    = false;
+    private $_useId   = true;
+    private $_srclang = null;
     private $_tu      = null;
     private $_tuv     = null;
     private $_seg     = null;
     private $_content = null;
     private $_data    = array();
 
-    /**
-     * Generates the tmx adapter
-     * This adapter reads with php's xml_parser
-     *
-     * @param  string              $data     Translation data
-     * @param  string|Zend_Locale  $locale   OPTIONAL Locale/Language to set, identical with locale identifier,
-     *                                       see Zend_Locale for more information
-     * @param  array               $options  OPTIONAL Options to set
-     */
-    public function __construct($data, $locale = null, array $options = array())
-    {
-        parent::__construct($data, $locale, $options);
-    }
-
-
     /**
      * Load translation data (TMX file reader)
      *
@@ -75,6 +67,10 @@ class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
             throw new Zend_Translate_Exception('Translation file \'' . $filename . '\' is not readable.');
         }
 
+        if (isset($options['useId'])) {
+            $this->_useId = (boolean) $options['useId'];
+        }
+
         $encoding = $this->_findEncoding($filename);
         $this->_file = xml_parser_create($encoding);
         xml_set_object($this->_file, $this);
@@ -82,10 +78,20 @@ class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
         xml_set_element_handler($this->_file, "_startElement", "_endElement");
         xml_set_character_data_handler($this->_file, "_contentElement");
 
+        try {
+            Zend_Xml_Security::scanFile($filename);
+        } catch (Zend_Xml_Exception $e) {
+            require_once 'Zend/Translate/Exception.php';
+            throw new Zend_Translate_Exception(
+                $e->getMessage()
+            );
+        }
+ 
         if (!xml_parse($this->_file, file_get_contents($filename))) {
-            $ex = sprintf('XML error: %s at line %d',
+            $ex = sprintf('XML error: %s at line %d of file %s',
                           xml_error_string(xml_get_error_code($this->_file)),
-                          xml_get_current_line_number($this->_file));
+                          xml_get_current_line_number($this->_file),
+                          $filename);
             xml_parser_free($this->_file);
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception($ex);
@@ -94,7 +100,14 @@ class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
         return $this->_data;
     }
 
-    private function _startElement($file, $name, $attrib)
+    /**
+     * Internal method, called by xml element handler at start
+     *
+     * @param resource $file   File handler
+     * @param string   $name   Elements name
+     * @param array    $attrib Attributes for this element
+     */
+    protected function _startElement($file, $name, $attrib)
     {
         if ($this->_seg !== null) {
             $this->_content .= "<".$name;
@@ -104,15 +117,45 @@ class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
             $this->_content .= ">";
         } else {
             switch(strtolower($name)) {
+                case 'header':
+                    if (empty($this->_useId) && isset($attrib['srclang'])) {
+                        if (Zend_Locale::isLocale($attrib['srclang'])) {
+                            $this->_srclang = Zend_Locale::findLocale($attrib['srclang']);
+                        } else {
+                            if (!$this->_options['disableNotices']) {
+                                if ($this->_options['log']) {
+                                    $this->_options['log']->notice("The language '{$attrib['srclang']}' can not be set because it does not exist.");
+                                } else {
+                                    trigger_error("The language '{$attrib['srclang']}' can not be set because it does not exist.", E_USER_NOTICE);
+                                }
+                            }
+
+                            $this->_srclang = $attrib['srclang'];
+                        }
+                    }
+                    break;
                 case 'tu':
-                    if (isset($attrib['tuid']) === true) {
+                    if (isset($attrib['tuid'])) {
                         $this->_tu = $attrib['tuid'];
                     }
                     break;
                 case 'tuv':
-                    if (isset($attrib['xml:lang']) === true) {
-                        $this->_tuv = $attrib['xml:lang'];
-                        if (isset($this->_data[$this->_tuv]) === false) {
+                    if (isset($attrib['xml:lang'])) {
+                        if (Zend_Locale::isLocale($attrib['xml:lang'])) {
+                            $this->_tuv = Zend_Locale::findLocale($attrib['xml:lang']);
+                        } else {
+                            if (!$this->_options['disableNotices']) {
+                                if ($this->_options['log']) {
+                                    $this->_options['log']->notice("The language '{$attrib['xml:lang']}' can not be set because it does not exist.");
+                                } else {
+                                    trigger_error("The language '{$attrib['xml:lang']}' can not be set because it does not exist.", E_USER_NOTICE);
+                                }
+                            }
+
+                            $this->_tuv = $attrib['xml:lang'];
+                        }
+
+                        if (!isset($this->_data[$this->_tuv])) {
                             $this->_data[$this->_tuv] = array();
                         }
                     }
@@ -127,7 +170,14 @@ class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
         }
     }
 
-    private function _endElement($file, $name)
+
+    /**
+     * Internal method, called by xml element handler at end
+     *
+     * @param resource $file   File handler
+     * @param string   $name   Elements name
+     */
+    protected function _endElement($file, $name)
     {
         if (($this->_seg !== null) and ($name !== 'seg')) {
             $this->_content .= "</".$name.">";
@@ -141,6 +191,10 @@ class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
                     break;
                 case 'seg':
                     $this->_seg = null;
+                    if (!empty($this->_srclang) && ($this->_srclang == $this->_tuv)) {
+                        $this->_tu = $this->_content;
+                    }
+
                     if (!empty($this->_content) or (!isset($this->_data[$this->_tuv][$this->_tu]))) {
                         $this->_data[$this->_tuv][$this->_tu] = $this->_content;
                     }
@@ -151,14 +205,27 @@ class Zend_Translate_Adapter_Tmx extends Zend_Translate_Adapter {
         }
     }
 
-    private function _contentElement($file, $data)
+    /**
+     * Internal method, called by xml element handler for content
+     *
+     * @param resource $file File handler
+     * @param string   $data Elements content
+     */
+    protected function _contentElement($file, $data)
     {
         if (($this->_seg !== null) and ($this->_tu !== null) and ($this->_tuv !== null)) {
             $this->_content .= $data;
         }
     }
 
-    private function _findEncoding($filename)
+
+    /**
+     * Internal method, detects the encoding of the xml file
+     *
+     * @param string $name Filename
+     * @return string Encoding
+     */
+    protected function _findEncoding($filename)
     {
         $file = file_get_contents($filename, null, null, 0, 100);
         if (strpos($file, "encoding") !== false) {
