@@ -14,9 +14,9 @@
  *
  * @category   Zend
  * @package    Zend_Validate
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Hostname.php 21063 2010-02-15 23:00:17Z thomas $
+ * @version    $Id$
  */
 
 /**
@@ -41,36 +41,38 @@ require_once 'Zend/Validate/Ip.php';
  *
  * @category   Zend
  * @package    Zend_Validate
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Validate_Hostname extends Zend_Validate_Abstract
 {
-    const INVALID                 = 'hostnameInvalid';
-    const IP_ADDRESS_NOT_ALLOWED  = 'hostnameIpAddressNotAllowed';
-    const UNKNOWN_TLD             = 'hostnameUnknownTld';
-    const INVALID_DASH            = 'hostnameDashCharacter';
-    const INVALID_HOSTNAME_SCHEMA = 'hostnameInvalidHostnameSchema';
-    const UNDECIPHERABLE_TLD      = 'hostnameUndecipherableTld';
-    const INVALID_HOSTNAME        = 'hostnameInvalidHostname';
-    const INVALID_LOCAL_NAME      = 'hostnameInvalidLocalName';
-    const LOCAL_NAME_NOT_ALLOWED  = 'hostnameLocalNameNotAllowed';
     const CANNOT_DECODE_PUNYCODE  = 'hostnameCannotDecodePunycode';
+    const INVALID                 = 'hostnameInvalid';
+    const INVALID_DASH            = 'hostnameDashCharacter';
+    const INVALID_HOSTNAME        = 'hostnameInvalidHostname';
+    const INVALID_HOSTNAME_SCHEMA = 'hostnameInvalidHostnameSchema';
+    const INVALID_LOCAL_NAME      = 'hostnameInvalidLocalName';
+    const INVALID_URI             = 'hostnameInvalidUri';
+    const IP_ADDRESS_NOT_ALLOWED  = 'hostnameIpAddressNotAllowed';
+    const LOCAL_NAME_NOT_ALLOWED  = 'hostnameLocalNameNotAllowed';
+    const UNDECIPHERABLE_TLD      = 'hostnameUndecipherableTld';
+    const UNKNOWN_TLD             = 'hostnameUnknownTld';
 
     /**
      * @var array
      */
     protected $_messageTemplates = array(
-        self::INVALID                 => "Invalid type given, value should be a string",
-        self::IP_ADDRESS_NOT_ALLOWED  => "'%value%' appears to be an IP address, but IP addresses are not allowed",
-        self::UNKNOWN_TLD             => "'%value%' appears to be a DNS hostname but cannot match TLD against known list",
-        self::INVALID_DASH            => "'%value%' appears to be a DNS hostname but contains a dash in an invalid position",
-        self::INVALID_HOSTNAME_SCHEMA => "'%value%' appears to be a DNS hostname but cannot match against hostname schema for TLD '%tld%'",
-        self::UNDECIPHERABLE_TLD      => "'%value%' appears to be a DNS hostname but cannot extract TLD part",
-        self::INVALID_HOSTNAME        => "'%value%' does not match the expected structure for a DNS hostname",
-        self::INVALID_LOCAL_NAME      => "'%value%' does not appear to be a valid local network name",
-        self::LOCAL_NAME_NOT_ALLOWED  => "'%value%' appears to be a local network name but local network names are not allowed",
         self::CANNOT_DECODE_PUNYCODE  => "'%value%' appears to be a DNS hostname but the given punycode notation cannot be decoded",
+        self::INVALID                 => "Invalid type given. String expected",
+        self::INVALID_DASH            => "'%value%' appears to be a DNS hostname but contains a dash in an invalid position",
+        self::INVALID_HOSTNAME        => "'%value%' does not match the expected structure for a DNS hostname",
+        self::INVALID_HOSTNAME_SCHEMA => "'%value%' appears to be a DNS hostname but cannot match against hostname schema for TLD '%tld%'",
+        self::INVALID_LOCAL_NAME      => "'%value%' does not appear to be a valid local network name",
+        self::INVALID_URI             => "'%value%' does not appear to be a valid URI hostname",
+        self::IP_ADDRESS_NOT_ALLOWED  => "'%value%' appears to be an IP address, but IP addresses are not allowed",
+        self::LOCAL_NAME_NOT_ALLOWED  => "'%value%' appears to be a local network name but local network names are not allowed",
+        self::UNDECIPHERABLE_TLD      => "'%value%' appears to be a DNS hostname but cannot extract TLD part",
+        self::UNKNOWN_TLD             => "'%value%' appears to be a DNS hostname but cannot match TLD against known list",
     );
 
     /**
@@ -98,35 +100,662 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
     /**
      * Allows all types of hostnames
      */
-    const ALLOW_ALL   = 7;
+    const ALLOW_URI = 8;
+
+    /**
+     * Allows all types of hostnames
+     */
+    const ALLOW_ALL = 15;
 
     /**
      * Array of valid top-level-domains
      *
-     * @see ftp://data.iana.org/TLD/tlds-alpha-by-domain.txt  List of all TLDs by domain
+     * Version 2014071001, Last Updated Fri Jul 11 07:07:01 2014 UTC
+     *
+     * @see http://data.iana.org/TLD/tlds-alpha-by-domain.txt  List of all TLDs by domain
      * @see http://www.iana.org/domains/root/db/ Official list of supported TLDs
      * @var array
      */
     protected $_validTlds = array(
-        'ac', 'ad', 'ae', 'aero', 'af', 'ag', 'ai', 'al', 'am', 'an', 'ao', 'aq', 'ar', 'arpa',
-        'as', 'asia', 'at', 'au', 'aw', 'ax', 'az', 'ba', 'bb', 'bd', 'be', 'bf', 'bg', 'bh', 'bi',
-        'biz', 'bj', 'bm', 'bn', 'bo', 'br', 'bs', 'bt', 'bv', 'bw', 'by', 'bz', 'ca', 'cat', 'cc',
-        'cd', 'cf', 'cg', 'ch', 'ci', 'ck', 'cl', 'cm', 'cn', 'co', 'com', 'coop', 'cr', 'cu',
-        'cv', 'cx', 'cy', 'cz', 'de', 'dj', 'dk', 'dm', 'do', 'dz', 'ec', 'edu', 'ee', 'eg', 'er',
-        'es', 'et', 'eu', 'fi', 'fj', 'fk', 'fm', 'fo', 'fr', 'ga', 'gb', 'gd', 'ge', 'gf', 'gg',
-        'gh', 'gi', 'gl', 'gm', 'gn', 'gov', 'gp', 'gq', 'gr', 'gs', 'gt', 'gu', 'gw', 'gy', 'hk',
-        'hm', 'hn', 'hr', 'ht', 'hu', 'id', 'ie', 'il', 'im', 'in', 'info', 'int', 'io', 'iq',
-        'ir', 'is', 'it', 'je', 'jm', 'jo', 'jobs', 'jp', 'ke', 'kg', 'kh', 'ki', 'km', 'kn', 'kp',
-        'kr', 'kw', 'ky', 'kz', 'la', 'lb', 'lc', 'li', 'lk', 'lr', 'ls', 'lt', 'lu', 'lv', 'ly',
-        'ma', 'mc', 'md', 'me', 'mg', 'mh', 'mil', 'mk', 'ml', 'mm', 'mn', 'mo', 'mobi', 'mp',
-        'mq', 'mr', 'ms', 'mt', 'mu', 'museum', 'mv', 'mw', 'mx', 'my', 'mz', 'na', 'name', 'nc',
-        'ne', 'net', 'nf', 'ng', 'ni', 'nl', 'no', 'np', 'nr', 'nu', 'nz', 'om', 'org', 'pa', 'pe',
-        'pf', 'pg', 'ph', 'pk', 'pl', 'pm', 'pn', 'pr', 'pro', 'ps', 'pt', 'pw', 'py', 'qa', 're',
-        'ro', 'rs', 'ru', 'rw', 'sa', 'sb', 'sc', 'sd', 'se', 'sg', 'sh', 'si', 'sj', 'sk', 'sl',
-        'sm', 'sn', 'so', 'sr', 'st', 'su', 'sv', 'sy', 'sz', 'tc', 'td', 'tel', 'tf', 'tg', 'th',
-        'tj', 'tk', 'tl', 'tm', 'tn', 'to', 'tp', 'tr', 'travel', 'tt', 'tv', 'tw', 'tz', 'ua',
-        'ug', 'uk', 'um', 'us', 'uy', 'uz', 'va', 'vc', 've', 'vg', 'vi', 'vn', 'vu', 'wf', 'ws',
-        'ye', 'yt', 'yu', 'za', 'zm', 'zw'
+        'ac',
+        'academy',
+        'accountants',
+        'active',
+        'actor',
+        'ad',
+        'ae',
+        'aero',
+        'af',
+        'ag',
+        'agency',
+        'ai',
+        'airforce',
+        'al',
+        'am',
+        'an',
+        'ao',
+        'aq',
+        'ar',
+        'archi',
+        'army',
+        'arpa',
+        'as',
+        'asia',
+        'associates',
+        'at',
+        'attorney',
+        'au',
+        'audio',
+        'autos',
+        'aw',
+        'ax',
+        'axa',
+        'az',
+        'ba',
+        'bar',
+        'bargains',
+        'bayern',
+        'bb',
+        'bd',
+        'be',
+        'beer',
+        'berlin',
+        'best',
+        'bf',
+        'bg',
+        'bh',
+        'bi',
+        'bid',
+        'bike',
+        'bio',
+        'biz',
+        'bj',
+        'black',
+        'blackfriday',
+        'blue',
+        'bm',
+        'bmw',
+        'bn',
+        'bo',
+        'boutique',
+        'br',
+        'brussels',
+        'bs',
+        'bt',
+        'build',
+        'builders',
+        'buzz',
+        'bv',
+        'bw',
+        'by',
+        'bz',
+        'bzh',
+        'ca',
+        'cab',
+        'camera',
+        'camp',
+        'cancerresearch',
+        'capetown',
+        'capital',
+        'cards',
+        'care',
+        'career',
+        'careers',
+        'cash',
+        'cat',
+        'catering',
+        'cc',
+        'cd',
+        'center',
+        'ceo',
+        'cf',
+        'cg',
+        'ch',
+        'cheap',
+        'christmas',
+        'church',
+        'ci',
+        'citic',
+        'city',
+        'ck',
+        'cl',
+        'claims',
+        'cleaning',
+        'clinic',
+        'clothing',
+        'club',
+        'cm',
+        'cn',
+        'co',
+        'codes',
+        'coffee',
+        'college',
+        'cologne',
+        'com',
+        'community',
+        'company',
+        'computer',
+        'condos',
+        'construction',
+        'consulting',
+        'contractors',
+        'cooking',
+        'cool',
+        'coop',
+        'country',
+        'cr',
+        'credit',
+        'creditcard',
+        'cruises',
+        'cu',
+        'cuisinella',
+        'cv',
+        'cw',
+        'cx',
+        'cy',
+        'cz',
+        'dance',
+        'dating',
+        'de',
+        'deals',
+        'degree',
+        'democrat',
+        'dental',
+        'dentist',
+        'desi',
+        'diamonds',
+        'digital',
+        'direct',
+        'directory',
+        'discount',
+        'dj',
+        'dk',
+        'dm',
+        'dnp',
+        'do',
+        'domains',
+        'durban',
+        'dz',
+        'ec',
+        'edu',
+        'education',
+        'ee',
+        'eg',
+        'email',
+        'engineer',
+        'engineering',
+        'enterprises',
+        'equipment',
+        'er',
+        'es',
+        'estate',
+        'et',
+        'eu',
+        'eus',
+        'events',
+        'exchange',
+        'expert',
+        'exposed',
+        'fail',
+        'farm',
+        'feedback',
+        'fi',
+        'finance',
+        'financial',
+        'fish',
+        'fishing',
+        'fitness',
+        'fj',
+        'fk',
+        'flights',
+        'florist',
+        'fm',
+        'fo',
+        'foo',
+        'foundation',
+        'fr',
+        'frogans',
+        'fund',
+        'furniture',
+        'futbol',
+        'ga',
+        'gal',
+        'gallery',
+        'gb',
+        'gd',
+        'ge',
+        'gf',
+        'gg',
+        'gh',
+        'gi',
+        'gift',
+        'gives',
+        'gl',
+        'glass',
+        'global',
+        'globo',
+        'gm',
+        'gmo',
+        'gn',
+        'gop',
+        'gov',
+        'gp',
+        'gq',
+        'gr',
+        'graphics',
+        'gratis',
+        'green',
+        'gripe',
+        'gs',
+        'gt',
+        'gu',
+        'guide',
+        'guitars',
+        'guru',
+        'gw',
+        'gy',
+        'hamburg',
+        'haus',
+        'hiphop',
+        'hiv',
+        'hk',
+        'hm',
+        'hn',
+        'holdings',
+        'holiday',
+        'homes',
+        'horse',
+        'host',
+        'house',
+        'hr',
+        'ht',
+        'hu',
+        'id',
+        'ie',
+        'il',
+        'im',
+        'immobilien',
+        'in',
+        'industries',
+        'info',
+        'ink',
+        'institute',
+        'insure',
+        'int',
+        'international',
+        'investments',
+        'io',
+        'iq',
+        'ir',
+        'is',
+        'it',
+        'je',
+        'jetzt',
+        'jm',
+        'jo',
+        'jobs',
+        'joburg',
+        'jp',
+        'juegos',
+        'kaufen',
+        'ke',
+        'kg',
+        'kh',
+        'ki',
+        'kim',
+        'kitchen',
+        'kiwi',
+        'km',
+        'kn',
+        'koeln',
+        'kp',
+        'kr',
+        'kred',
+        'kw',
+        'ky',
+        'kz',
+        'la',
+        'land',
+        'lawyer',
+        'lb',
+        'lc',
+        'lease',
+        'li',
+        'life',
+        'lighting',
+        'limited',
+        'limo',
+        'link',
+        'lk',
+        'loans',
+        'london',
+        'lotto',
+        'lr',
+        'ls',
+        'lt',
+        'lu',
+        'luxe',
+        'luxury',
+        'lv',
+        'ly',
+        'ma',
+        'maison',
+        'management',
+        'mango',
+        'market',
+        'marketing',
+        'mc',
+        'md',
+        'me',
+        'media',
+        'meet',
+        'melbourne',
+        'menu',
+        'mg',
+        'mh',
+        'miami',
+        'mil',
+        'mini',
+        'mk',
+        'ml',
+        'mm',
+        'mn',
+        'mo',
+        'mobi',
+        'moda',
+        'moe',
+        'monash',
+        'mortgage',
+        'moscow',
+        'motorcycles',
+        'mp',
+        'mq',
+        'mr',
+        'ms',
+        'mt',
+        'mu',
+        'museum',
+        'mv',
+        'mw',
+        'mx',
+        'my',
+        'mz',
+        'na',
+        'nagoya',
+        'name',
+        'navy',
+        'nc',
+        'ne',
+        'net',
+        'neustar',
+        'nf',
+        'ng',
+        'nhk',
+        'ni',
+        'ninja',
+        'nl',
+        'no',
+        'np',
+        'nr',
+        'nu',
+        'nyc',
+        'nz',
+        'okinawa',
+        'om',
+        'onl',
+        'org',
+        'organic',
+        'ovh',
+        'pa',
+        'paris',
+        'partners',
+        'parts',
+        'pe',
+        'pf',
+        'pg',
+        'ph',
+        'photo',
+        'photography',
+        'photos',
+        'physio',
+        'pics',
+        'pictures',
+        'pink',
+        'pk',
+        'pl',
+        'place',
+        'plumbing',
+        'pm',
+        'pn',
+        'post',
+        'pr',
+        'press',
+        'pro',
+        'productions',
+        'properties',
+        'ps',
+        'pt',
+        'pub',
+        'pw',
+        'py',
+        'qa',
+        'qpon',
+        'quebec',
+        're',
+        'recipes',
+        'red',
+        'rehab',
+        'reise',
+        'reisen',
+        'ren',
+        'rentals',
+        'repair',
+        'report',
+        'republican',
+        'rest',
+        'reviews',
+        'rich',
+        'rio',
+        'ro',
+        'rocks',
+        'rodeo',
+        'rs',
+        'ru',
+        'ruhr',
+        'rw',
+        'ryukyu',
+        'sa',
+        'saarland',
+        'sb',
+        'sc',
+        'schmidt',
+        'schule',
+        'scot',
+        'sd',
+        'se',
+        'services',
+        'sexy',
+        'sg',
+        'sh',
+        'shiksha',
+        'shoes',
+        'si',
+        'singles',
+        'sj',
+        'sk',
+        'sl',
+        'sm',
+        'sn',
+        'so',
+        'social',
+        'software',
+        'sohu',
+        'solar',
+        'solutions',
+        'soy',
+        'space',
+        'sr',
+        'st',
+        'su',
+        'supplies',
+        'supply',
+        'support',
+        'surf',
+        'surgery',
+        'suzuki',
+        'sv',
+        'sx',
+        'sy',
+        'systems',
+        'sz',
+        'tattoo',
+        'tax',
+        'tc',
+        'td',
+        'technology',
+        'tel',
+        'tf',
+        'tg',
+        'th',
+        'tienda',
+        'tips',
+        'tirol',
+        'tj',
+        'tk',
+        'tl',
+        'tm',
+        'tn',
+        'to',
+        'today',
+        'tokyo',
+        'tools',
+        'town',
+        'toys',
+        'tp',
+        'tr',
+        'trade',
+        'training',
+        'travel',
+        'tt',
+        'tv',
+        'tw',
+        'tz',
+        'ua',
+        'ug',
+        'uk',
+        'university',
+        'uno',
+        'us',
+        'uy',
+        'uz',
+        'va',
+        'vacations',
+        'vc',
+        've',
+        'vegas',
+        'ventures',
+        'versicherung',
+        'vet',
+        'vg',
+        'vi',
+        'viajes',
+        'villas',
+        'vision',
+        'vlaanderen',
+        'vn',
+        'vodka',
+        'vote',
+        'voting',
+        'voto',
+        'voyage',
+        'vu',
+        'wang',
+        'watch',
+        'webcam',
+        'website',
+        'wed',
+        'wf',
+        'wien',
+        'wiki',
+        'works',
+        'ws',
+        'wtc',
+        'wtf',
+        'xn--3bst00m',
+        'xn--3ds443g',
+        'xn--3e0b707e',
+        'xn--45brj9c',
+        'xn--4gbrim',
+        'xn--55qw42g',
+        'xn--55qx5d',
+        'xn--6frz82g',
+        'xn--6qq986b3xl',
+        'xn--80adxhks',
+        'xn--80ao21a',
+        'xn--80asehdb',
+        'xn--80aswg',
+        'xn--90a3ac',
+        'xn--c1avg',
+        'xn--cg4bki',
+        'xn--clchc0ea0b2g2a9gcd',
+        'xn--czr694b',
+        'xn--czru2d',
+        'xn--d1acj3b',
+        'xn--fiq228c5hs',
+        'xn--fiq64b',
+        'xn--fiqs8s',
+        'xn--fiqz9s',
+        'xn--fpcrj9c3d',
+        'xn--fzc2c9e2c',
+        'xn--gecrj9c',
+        'xn--h2brj9c',
+        'xn--i1b6b1a6a2e',
+        'xn--io0a7i',
+        'xn--j1amh',
+        'xn--j6w193g',
+        'xn--kprw13d',
+        'xn--kpry57d',
+        'xn--kput3i',
+        'xn--l1acc',
+        'xn--lgbbat1ad8j',
+        'xn--mgb9awbf',
+        'xn--mgba3a4f16a',
+        'xn--mgbaam7a8h',
+        'xn--mgbab2bd',
+        'xn--mgbayh7gpa',
+        'xn--mgbbh1a71e',
+        'xn--mgbc0a9azcg',
+        'xn--mgberp4a5d4ar',
+        'xn--mgbx4cd0ab',
+        'xn--ngbc5azd',
+        'xn--nqv7f',
+        'xn--nqv7fs00ema',
+        'xn--o3cw4h',
+        'xn--ogbpf8fl',
+        'xn--p1ai',
+        'xn--pgbs0dh',
+        'xn--q9jyb4c',
+        'xn--rhqv96g',
+        'xn--s9brj9c',
+        'xn--ses554g',
+        'xn--unup4y',
+        'xn--wgbh1c',
+        'xn--wgbl6a',
+        'xn--xkc2al3hye2a',
+        'xn--xkc2dl3a5ee0h',
+        'xn--yfro4i67o',
+        'xn--ygbi2ammx',
+        'xn--zfr164b',
+        'xxx',
+        'xyz',
+        'yachts',
+        'ye',
+        'yokohama',
+        'yt',
+        'za',
+        'zm',
+        'zone',
+        'zw',
     );
 
     /**
@@ -144,6 +773,7 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
      * (.BIZ) International http://www.iana.org/domains/idn-tables/
      * (.BR) Brazil http://registro.br/faq/faq6.html
      * (.BV) Bouvett Island http://www.norid.no/domeneregistrering/idn/idn_nyetegn.en.html
+     * (.CA) Canada http://www.iana.org/domains/idn-tables/tables/ca_fr_1.0.html
      * (.CAT) Catalan http://www.iana.org/domains/idn-tables/tables/cat_ca_1.0.html
      * (.CH) Switzerland https://nic.switch.ch/reg/ocView.action?res=EF6GW2JBPVTG67DLNIQXU234MN6SC33JNQQGI7L6#anhang1
      * (.CL) Chile http://www.iana.org/domains/idn-tables/tables/cl_latn_1.0.html
@@ -172,6 +802,7 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
      * (.PR) Puerto Rico http://www.nic.pr/idn_rules.asp
      * (.PT) Portugal https://online.dns.pt/dns_2008/do?com=DS;8216320233;111;+PAGE(4000058)+K-CAT-CODIGO(C.125)+RCNT(100);
      * (.RU) Russia http://www.iana.org/domains/idn-tables/tables/ru_ru-ru_1.0.html
+     * (.RS) Serbia http://www.iana.org/domains/idn-tables/tables/rs_sr-rs_1.0.pdf
      * (.SA) Saudi Arabia http://www.iana.org/domains/idn-tables/tables/sa_ar_1.0.html
      * (.SE) Sweden http://www.iis.se/english/IDN_campaignsite.shtml?lang=en
      * (.SH) Saint Helena http://www.nic.sh/SH-IDN-Policy.pdf
@@ -179,6 +810,7 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
      * (.TH) Thailand http://www.iana.org/domains/idn-tables/tables/th_th-th_1.0.html
      * (.TM) Turkmenistan http://www.nic.tm/TM-IDN-Policy.pdf
      * (.TR) Turkey https://www.nic.tr/index.php
+     * (.UA) Ukraine http://www.iana.org/domains/idn-tables/tables/ua_cyrl_1.2.html
      * (.VE) Venice http://www.iana.org/domains/idn-tables/tables/ve_es_1.0.html
      * (.VN) Vietnam http://www.vnnic.vn/english/5-6-300-2-2-04-20071115.htm#1.%20Introduction
      *
@@ -192,13 +824,14 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
         'BIZ' => 'Hostname/Biz.php',
         'BR'  => array(1 => '/^[\x{002d}0-9a-zà-ãçéíó-õúü]{1,63}$/iu'),
         'BV'  => array(1 => '/^[\x{002d}0-9a-zàáä-éêñ-ôöøüčđńŋšŧž]{1,63}$/iu'),
+        'CA'  => array(1 => '/^[\x{002d}0-9a-zàâæçéèêëîïôœùûüÿ\x{00E0}\x{00E2}\x{00E7}\x{00E8}\x{00E9}\x{00EA}\x{00EB}\x{00EE}\x{00EF}\x{00F4}\x{00F9}\x{00FB}\x{00FC}\x{00E6}\x{0153}\x{00FF}]{1,63}$/iu'),
         'CAT' => array(1 => '/^[\x{002d}0-9a-z·àç-éíïòóúü]{1,63}$/iu'),
         'CH'  => array(1 => '/^[\x{002d}0-9a-zà-öø-ÿœ]{1,63}$/iu'),
         'CL'  => array(1 => '/^[\x{002d}0-9a-záéíñóúü]{1,63}$/iu'),
         'CN'  => 'Hostname/Cn.php',
         'COM' => 'Zend/Validate/Hostname/Com.php',
         'DE'  => array(1 => '/^[\x{002d}0-9a-zà-öø-ÿăąāćĉčċďđĕěėęēğĝġģĥħĭĩįīıĵķĺľļłńňņŋŏőōœĸŕřŗśŝšşťţŧŭůűũųūŵŷźžż]{1,63}$/iu'),
-        'DK'  => array(1 => '/^[\x{002d}0-9a-zäéöü]{1,63}$/iu'),
+        'DK'  => array(1 => '/^[\x{002d}0-9a-zäéöüæøå]{1,63}$/iu'),
         'ES'  => array(1 => '/^[\x{002d}0-9a-zàáçèéíïñòóúü·]{1,63}$/iu'),
         'EU'  => array(1 => '/^[\x{002d}0-9a-zà-öø-ÿ]{1,63}$/iu',
             2 => '/^[\x{002d}0-9a-zāăąćĉċčďđēĕėęěĝğġģĥħĩīĭįıĵķĺļľŀłńņňŉŋōŏőœŕŗřśŝšťŧũūŭůűųŵŷźżž]{1,63}$/iu',
@@ -272,15 +905,21 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
             33=> '/^[\x{002d}0-9א-ת]{1,63}$/iu'),
         'PR'  => array(1 => '/^[\x{002d}0-9a-záéíóúñäëïüöâêîôûàèùæçœãõ]{1,63}$/iu'),
         'PT'  => array(1 => '/^[\x{002d}0-9a-záàâãçéêíóôõú]{1,63}$/iu'),
+        'RS'  => array(1 => '/^[\x{002D}\x{0030}-\x{0039}\x{0061}-\x{007A}\x{0107}\x{010D}\x{0111}\x{0161}\x{017E}]{1,63}$/iu)'),
         'RU'  => array(1 => '/^[\x{002d}0-9а-яё]{1,63}$/iu'),
         'SA'  => array(1 => '/^[\x{002d}.0-9\x{0621}-\x{063A}\x{0641}-\x{064A}\x{0660}-\x{0669}]{1,63}$/iu'),
         'SE'  => array(1 => '/^[\x{002d}0-9a-zäåéöü]{1,63}$/iu'),
         'SH'  => array(1 => '/^[\x{002d}0-9a-zà-öø-ÿăąāćĉčċďđĕěėęēğĝġģĥħĭĩįīıĵķĺľļłńňņŋŏőōœĸŕřŗśŝšşťţŧŭůűũųūŵŷźžż]{1,63}$/iu'),
+        'SI'  => array(
+            1 => '/^[\x{002d}0-9a-zà-öø-ÿ]{1,63}$/iu',
+            2 => '/^[\x{002d}0-9a-zāăąćĉċčďđēĕėęěĝğġģĥħĩīĭįıĵķĺļľŀłńņňŉŋōŏőœŕŗřśŝšťŧũūŭůűųŵŷźżž]{1,63}$/iu',
+            3 => '/^[\x{002d}0-9a-zșț]{1,63}$/iu'),
         'SJ'  => array(1 => '/^[\x{002d}0-9a-zàáä-éêñ-ôöøüčđńŋšŧž]{1,63}$/iu'),
         'TH'  => array(1 => '/^[\x{002d}0-9a-z\x{0E01}-\x{0E3A}\x{0E40}-\x{0E4D}\x{0E50}-\x{0E59}]{1,63}$/iu'),
         'TM'  => array(1 => '/^[\x{002d}0-9a-zà-öø-ÿāăąćĉċčďđēėęěĝġģĥħīįĵķĺļľŀłńņňŋőœŕŗřśŝşšţťŧūŭůűųŵŷźżž]{1,63}$/iu'),
         'TW'  => 'Zend/Validate/Hostname/Cn.php',
         'TR'  => array(1 => '/^[\x{002d}0-9a-zğıüşöç]{1,63}$/iu'),
+        'UA'  => array(1 => '/^[\x{002d}0-9a-zабвгдежзийклмнопрстуфхцчшщъыьэюяѐёђѓєѕіїјљњћќѝўџґӂʼ]{1,63}$/iu'),
         'VE'  => array(1 => '/^[\x{002d}0-9a-záéíóúüñ]{1,63}$/iu'),
         'VN'  => array(1 => '/^[ÀÁÂÃÈÉÊÌÍÒÓÔÕÙÚÝàáâãèéêìíòóôõùúýĂăĐđĨĩŨũƠơƯư\x{1EA0}-\x{1EF9}]{1,63}$/iu'),
         'ایران' => array(1 => '/^[\x{0621}-\x{0624}\x{0626}-\x{063A}\x{0641}\x{0642}\x{0644}-\x{0648}\x{067E}\x{0686}\x{0698}\x{06A9}\x{06AF}\x{06CC}\x{06F0}-\x{06F9}]{1,30}$/iu'),
@@ -495,7 +1134,7 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
 
         $this->_setValue($value);
         // Check input against IP address schema
-        if (preg_match('/^[0-9.a-e:.]*$/i', $value) &&
+        if (preg_match('/^[0-9a-f:.]*$/i', $value) &&
             $this->_options['ip']->setTranslator($this->getTranslator())->isValid($value)) {
             if (!($this->_options['allow'] & self::ALLOW_IP)) {
                 $this->_error(self::IP_ADDRESS_NOT_ALLOWED);
@@ -505,17 +1144,51 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
             }
         }
 
-        // Check input against DNS hostname schema
+        // RFC3986 3.2.2 states:
+        //
+        //     The rightmost domain label of a fully qualified domain name
+        //     in DNS may be followed by a single "." and should be if it is
+        //     necessary to distinguish between the complete domain name and
+        //     some local domain.
+        //
+        // (see ZF-6363)
+
+        // Local hostnames are allowed to be partitial (ending '.')
+        if ($this->_options['allow'] & self::ALLOW_LOCAL) {
+            if (substr($value, -1) === '.') {
+                $value = substr($value, 0, -1);
+                if (substr($value, -1) === '.') {
+                    // Empty hostnames (ending '..') are not allowed
+                    $this->_error(self::INVALID_LOCAL_NAME);
+                    return false;
+                }
+            }
+        }
+
         $domainParts = explode('.', $value);
+
+        // Prevent partitial IP V4 adresses (ending '.')
+        if ((count($domainParts) == 4) && preg_match('/^[0-9.a-e:.]*$/i', $value) &&
+            $this->_options['ip']->setTranslator($this->getTranslator())->isValid($value)) {
+            $this->_error(self::INVALID_LOCAL_NAME);
+        }
+
+        // Check input against DNS hostname schema
         if ((count($domainParts) > 1) && (strlen($value) >= 4) && (strlen($value) <= 254)) {
             $status = false;
 
-            $origenc = iconv_get_encoding('internal_encoding');
-            iconv_set_encoding('internal_encoding', 'UTF-8');
+            $origenc = PHP_VERSION_ID < 50600
+                        ? iconv_get_encoding('internal_encoding')
+                        : ini_get('default_charset');
+            if (PHP_VERSION_ID < 50600) {
+                iconv_set_encoding('internal_encoding', 'UTF-8');
+            } else {
+                ini_set('default_charset', 'UTF-8');
+            }
             do {
                 // First check TLD
                 $matches = array();
-                if (preg_match('/([^.]{2,10})$/i', end($domainParts), $matches) ||
+                if (preg_match('/([^.]{2,63})$/i', end($domainParts), $matches) ||
                     (end($domainParts) == 'ایران') || (end($domainParts) == '中国') ||
                     (end($domainParts) == '公司') || (end($domainParts) == '网络')) {
 
@@ -553,6 +1226,12 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
                     // Check each hostname part
                     $check = 0;
                     foreach ($domainParts as $domainPart) {
+                        // If some domain part is empty (i.e. zend..com), it's invalid
+                        if (empty($domainPart)) {
+                            $this->_error(self::INVALID_HOSTNAME);
+                            return false;
+                        }
+
                         // Decode Punycode domainnames to IDN
                         if (strpos($domainPart, 'xn--') === 0) {
                             $domainPart = $this->decodePunycode(substr($domainPart, 4));
@@ -607,7 +1286,11 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
                 }
             } while (false);
 
-            iconv_set_encoding('internal_encoding', $origenc);
+            if (PHP_VERSION_ID < 50600) {
+                iconv_set_encoding('internal_encoding', $origenc);
+            } else {
+                ini_set('default_charset', $origenc);
+            }
             // If the input passes as an Internet domain name, and domain names are allowed, then the hostname
             // passes validation
             if ($status && ($this->_options['allow'] & self::ALLOW_DNS)) {
@@ -617,8 +1300,17 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
             $this->_error(self::INVALID_HOSTNAME);
         }
 
+        // Check for URI Syntax (RFC3986)
+        if ($this->_options['allow'] & self::ALLOW_URI) {
+            if (preg_match("/^([a-zA-Z0-9-._~!$&\'()*+,;=]|%[[:xdigit:]]{2}){1,254}$/i", $value)) {
+                return true;
+            } else {
+                $this->_error(self::INVALID_URI);
+            }
+        }
+
         // Check input against local network name schema; last chance to pass validation
-        $regexLocal = '/^(([a-zA-Z0-9\x2d]{1,63}\x2e)*[a-zA-Z0-9\x2d]{1,63}){1,254}$/';
+        $regexLocal = '/^(([a-zA-Z0-9\x2d]{1,63}\x2e)*[a-zA-Z0-9\x2d]{1,63}[\x2e]{0,1}){1,254}$/';
         $status = @preg_match($regexLocal, $value);
 
         // If the input passes as a local network name, and local network names are allowed, then the
@@ -650,22 +1342,19 @@ class Zend_Validate_Hostname extends Zend_Validate_Abstract
      */
     protected function decodePunycode($encoded)
     {
-        $found = preg_match('/([^a-z0-9\x2d]{1,10})$/i', $encoded);
-        if (empty($encoded) || ($found > 0)) {
-            // no punycode encoded string, return as is
+        if (!preg_match('/^[a-z0-9-]+$/i', $encoded)) {
+            // no punycode encoded string
             $this->_error(self::CANNOT_DECODE_PUNYCODE);
             return false;
         }
 
+        $decoded = array();
         $separator = strrpos($encoded, '-');
         if ($separator > 0) {
             for ($x = 0; $x < $separator; ++$x) {
                 // prepare decoding matrix
                 $decoded[] = ord($encoded[$x]);
             }
-        } else {
-            $this->_error(self::CANNOT_DECODE_PUNYCODE);
-            return false;
         }
 
         $lengthd = count($decoded);
