@@ -15,8 +15,9 @@
  * @category   Zend
  * @package    Zend_XmlRpc
  * @subpackage Client
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id$
  */
 
 
@@ -26,18 +27,6 @@
  */
 require_once 'Zend/Http/Client.php';
 
-/**
- * Exception thrown when an HTTP error occurs
- * @see Zend_XmlRpc_Client_HttpException
- */
-require_once 'Zend/XmlRpc/Client/HttpException.php';
-
-/**
- * Exception thrown when an XML-RPC fault is returned
- * @see Zend_XmlRpc_Client_FaultException
- */
-require_once 'Zend/XmlRpc/Client/FaultException.php';
-
 /**
  * Enables object chaining for calling namespaced XML-RPC methods.
  * @see Zend_XmlRpc_Client_ServerProxy
@@ -82,7 +71,7 @@ require_once 'Zend/XmlRpc/Fault.php';
  * @category   Zend
  * @package    Zend_XmlRpc
  * @subpackage Client
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_XmlRpc_Client
@@ -222,7 +211,7 @@ class Zend_XmlRpc_Client
     /**
      * Returns a proxy object for more convenient method calls
      *
-     * @param $namespace  Namespace to proxy or empty string for none
+     * @param string $namespace  Namespace to proxy or empty string for none
      * @return Zend_XmlRpc_Client_ServerProxy
      */
     public function getProxy($namespace = '')
@@ -236,8 +225,8 @@ class Zend_XmlRpc_Client
 
     /**
      * Set skip system lookup flag
-     * 
-     * @param  bool $flag 
+     *
+     * @param  bool $flag
      * @return Zend_XmlRpc_Client
      */
     public function setSkipSystemLookup($flag = true)
@@ -248,7 +237,7 @@ class Zend_XmlRpc_Client
 
     /**
      * Skip system lookup when determining if parameter should be array or struct?
-     * 
+     *
      * @return bool
      */
     public function skipSystemLookup()
@@ -262,29 +251,46 @@ class Zend_XmlRpc_Client
      * @param Zend_XmlRpc_Request $request
      * @param null|Zend_XmlRpc_Response $response
      * @return void
+     * @throws Zend_XmlRpc_Client_HttpException
      */
     public function doRequest($request, $response = null)
     {
         $this->_lastRequest = $request;
 
-        iconv_set_encoding('input_encoding', 'UTF-8');
-        iconv_set_encoding('output_encoding', 'UTF-8');
-        iconv_set_encoding('internal_encoding', 'UTF-8');
+        if (PHP_VERSION_ID < 50600) {
+            iconv_set_encoding('input_encoding', 'UTF-8');
+            iconv_set_encoding('output_encoding', 'UTF-8');
+            iconv_set_encoding('internal_encoding', 'UTF-8');
+        } else {
+            ini_set('input_encoding', 'UTF-8');
+            ini_set('output_encoding', 'UTF-8');
+            ini_set('default_charset', 'UTF-8');
+        }
 
         $http = $this->getHttpClient();
-        $http->setUri($this->_serverAddress);
+        if($http->getUri() === null) {
+            $http->setUri($this->_serverAddress);
+        }
 
         $http->setHeaders(array(
             'Content-Type: text/xml; charset=utf-8',
-            'User-Agent: Zend_XmlRpc_Client',
             'Accept: text/xml',
         ));
 
+        if ($http->getHeader('user-agent') === null) {
+            $http->setHeaders(array('User-Agent: Zend_XmlRpc_Client'));
+        }
+
         $xml = $this->_lastRequest->__toString();
         $http->setRawData($xml);
         $httpResponse = $http->request(Zend_Http_Client::POST);
 
         if (! $httpResponse->isSuccessful()) {
+            /**
+             * Exception thrown when an HTTP error occurs
+             * @see Zend_XmlRpc_Client_HttpException
+             */
+            require_once 'Zend/XmlRpc/Client/HttpException.php';
             throw new Zend_XmlRpc_Client_HttpException(
                                     $httpResponse->getMessage(),
                                     $httpResponse->getStatus());
@@ -294,15 +300,16 @@ class Zend_XmlRpc_Client
             $response = new Zend_XmlRpc_Response();
         }
         $this->_lastResponse = $response;
-        $this->_lastResponse->loadXml($httpResponse->getBody());
+        $this->_lastResponse->loadXml(trim($httpResponse->getBody()));
     }
 
     /**
      * Send an XML-RPC request to the service (for a specific method)
      *
-     * @param string $method Name of the method we want to call
-     * @param array $params Array of parameters for the method
-     * @throws Zend_Http_Client_FaultException
+     * @param  string $method Name of the method we want to call
+     * @param  array $params Array of parameters for the method
+     * @return mixed
+     * @throws Zend_XmlRpc_Client_FaultException
      */
     public function call($method, $params=array())
     {
@@ -316,38 +323,81 @@ class Zend_XmlRpc_Client
                 $success = false;
             }
             if ($success) {
-                foreach ($params as $key => $param) {
-                    if (is_array($param) && empty($param)) {
-                        $type = 'array';
+                $validTypes = array(
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_ARRAY,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_BASE64,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_BOOLEAN,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_DATETIME,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_DOUBLE,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_I4,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_INTEGER,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_NIL,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_STRING,
+                    Zend_XmlRpc_Value::XMLRPC_TYPE_STRUCT,
+                );
+
+                if (!is_array($params)) {
+                    $params = array($params);
+                }
+
+                foreach ($params as $key => $param)
+                {
+                    if ($param instanceof Zend_XmlRpc_Value) {
+                        continue;
+                    }
+
+                    if (count($signatures) > 1) {
+                        $type = Zend_XmlRpc_Value::getXmlRpcTypeByValue($param);
                         foreach ($signatures as $signature) {
                             if (!is_array($signature)) {
                                 continue;
                             }
-                            if (array_key_exists($key + 1, $signature)) {
-                                $type = $signature[$key + 1];
-                                $type = (in_array($type, array('array', 'struct'))) ? $type : 'array';
-                                break;
+                            if (isset($signature['parameters'][$key])) {
+                                if ($signature['parameters'][$key] == $type) {
+                                    break;
+                                }
                             }
                         }
-                        $params[$key] = array(
-                            'type'  => $type, 
-                            'value' => $param
-                        );
-                    } 
+                    } elseif (isset($signatures[0]['parameters'][$key])) {
+                        $type = $signatures[0]['parameters'][$key];
+                    } else {
+                        $type = null;
+                    }
+
+                    if (empty($type) || !in_array($type, $validTypes)) {
+                        $type = Zend_XmlRpc_Value::AUTO_DETECT_TYPE;
+                    }
+
+                    $params[$key] = Zend_XmlRpc_Value::getXmlRpcValue($param, $type);
                 }
             }
         }
 
-        $request = new Zend_XmlRpc_Request($method, $params);
+        $request = $this->_createRequest($method, $params);
 
         $this->doRequest($request);
 
         if ($this->_lastResponse->isFault()) {
             $fault = $this->_lastResponse->getFault();
+            /**
+             * Exception thrown when an XML-RPC fault is returned
+             * @see Zend_XmlRpc_Client_FaultException
+             */
+            require_once 'Zend/XmlRpc/Client/FaultException.php';
             throw new Zend_XmlRpc_Client_FaultException($fault->getMessage(),
                                                         $fault->getCode());
         }
 
         return $this->_lastResponse->getReturnValue();
     }
+
+    /**
+     * Create request object
+     *
+     * @return Zend_XmlRpc_Request
+     */
+    protected function _createRequest($method, $params)
+    {
+        return new Zend_XmlRpc_Request($method, $params);
+    }
 }
