@@ -15,20 +15,15 @@
  * @category   Zend
  * @package    Zend_XmlRpc
  * @subpackage Server
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Server.php 10684 2008-08-05 16:05:15Z matthew $
+ * @version    $Id$
  */
 
 /**
- * Implement Zend_Server_Interface
+ * Extends Zend_Server_Abstract
  */
-require_once 'Zend/Server/Interface.php';
-
-/**
- * Exception this class throws
- */
-require_once 'Zend/XmlRpc/Server/Exception.php';
+require_once 'Zend/Server/Abstract.php';
 
 /**
  * XMLRPC Request
@@ -50,6 +45,11 @@ require_once 'Zend/XmlRpc/Response/Http.php';
  */
 require_once 'Zend/XmlRpc/Server/Fault.php';
 
+/**
+ * XMLRPC server system methods class
+ */
+require_once 'Zend/XmlRpc/Server/System.php';
+
 /**
  * Convert PHP to and from xmlrpc native types
  */
@@ -111,10 +111,10 @@ require_once 'Zend/Server/Reflection/Method.php';
  * @category   Zend
  * @package    Zend_XmlRpc
  * @subpackage Server
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
-class Zend_XmlRpc_Server implements Zend_Server_Interface
+class Zend_XmlRpc_Server extends Zend_Server_Abstract
 {
     /**
      * Character encoding
@@ -122,12 +122,6 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
      */
     protected $_encoding = 'UTF-8';
 
-    /**
-     * Array of dispatchables
-     * @var array
-     */
-    protected $_methods = array();
-
     /**
      * Request processed
      * @var null|Zend_XmlRpc_Request
@@ -142,40 +136,53 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
 
     /**
      * Dispatch table of name => method pairs
-     * @var array
+     * @var Zend_Server_Definition
      */
-    protected $_table = array();
+    protected $_table;
 
     /**
      * PHP types => XML-RPC types
      * @var array
      */
     protected $_typeMap = array(
-        'i4'               => 'i4',
-        'int'              => 'int',
-        'integer'          => 'int',
-        'double'           => 'double',
-        'float'            => 'double',
-        'real'             => 'double',
-        'boolean'          => 'boolean',
-        'bool'             => 'boolean',
-        'true'             => 'boolean',
-        'false'            => 'boolean',
-        'string'           => 'string',
-        'str'              => 'string',
-        'base64'           => 'base64',
-        'dateTime.iso8601' => 'dateTime.iso8601',
-        'date'             => 'dateTime.iso8601',
-        'time'             => 'dateTime.iso8601',
-        'time'             => 'dateTime.iso8601',
-        'array'            => 'array',
-        'struct'           => 'struct',
-        'null'             => 'nil',
-        'nil'              => 'nil',
-        'void'             => 'void',
-        'mixed'            => 'struct'
+        'i4'                         => 'i4',
+        'int'                        => 'int',
+        'integer'                    => 'int',
+        'Zend_Crypt_Math_BigInteger' => 'i8',
+        'i8'                         => 'i8',
+        'ex:i8'                      => 'i8',
+        'double'                     => 'double',
+        'float'                      => 'double',
+        'real'                       => 'double',
+        'boolean'                    => 'boolean',
+        'bool'                       => 'boolean',
+        'true'                       => 'boolean',
+        'false'                      => 'boolean',
+        'string'                     => 'string',
+        'str'                        => 'string',
+        'base64'                     => 'base64',
+        'dateTime.iso8601'           => 'dateTime.iso8601',
+        'date'                       => 'dateTime.iso8601',
+        'time'                       => 'dateTime.iso8601',
+        'time'                       => 'dateTime.iso8601',
+        'Zend_Date'                  => 'dateTime.iso8601',
+        'DateTime'                   => 'dateTime.iso8601',
+        'array'                      => 'array',
+        'struct'                     => 'struct',
+        'null'                       => 'nil',
+        'nil'                        => 'nil',
+        'ex:nil'                     => 'nil',
+        'void'                       => 'void',
+        'mixed'                      => 'struct',
     );
 
+    /**
+     * Send arguments to all methods or just constructor?
+     *
+     * @var bool
+     */
+    protected $_sendArgumentsToAllMethods = true;
+
     /**
      * Constructor
      *
@@ -185,110 +192,26 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
      */
     public function __construct()
     {
-        // Setup system.* methods
-        $system = array(
-            'listMethods',
-            'methodHelp',
-            'methodSignature',
-            'multicall'
-        );
+        $this->_table = new Zend_Server_Definition();
+        $this->_registerSystemMethods();
+    }
 
-        $class = Zend_Server_Reflection::reflectClass($this);
-        foreach ($system as $method) {
-            $reflection = new Zend_Server_Reflection_Method($class, new ReflectionMethod($this, $method), 'system');
-            $reflection->system = true;
-            $this->_methods[] = $reflection;
+    /**
+     * Proxy calls to system object
+     *
+     * @param  string $method
+     * @param  array $params
+     * @return mixed
+     * @throws Zend_XmlRpc_Server_Exception
+     */
+    public function __call($method, $params)
+    {
+        $system = $this->getSystem();
+        if (!method_exists($system, $method)) {
+            require_once 'Zend/XmlRpc/Server/Exception.php';
+            throw new Zend_XmlRpc_Server_Exception('Unknown instance method called on server: ' . $method);
         }
-
-        $this->_buildDispatchTable();
-    }
-
-    /**
-     * Map PHP parameter types to XML-RPC types
-     *
-     * @param Zend_Server_Reflection_Function_Abstract $method
-     * @return void
-     */
-    protected function _fixTypes(Zend_Server_Reflection_Function_Abstract $method)
-    {
-        foreach ($method->getPrototypes() as $prototype) {
-            foreach ($prototype->getParameters() as $param) {
-                $pType = $param->getType();
-                if (isset($this->_typeMap[$pType])) {
-                    $param->setType($this->_typeMap[$pType]);
-                } else {
-                    $param->setType('void');
-                }
-            }
-        }
-    }
-
-    /**
-     * Re/Build the dispatch table
-     *
-     * The dispatch table consists of a an array of method name =>
-     * Zend_Server_Reflection_Function_Abstract pairs
-     *
-     * @return void
-     */
-    protected function _buildDispatchTable()
-    {
-        $table      = array();
-        foreach ($this->_methods as $dispatchable) {
-            if ($dispatchable instanceof Zend_Server_Reflection_Function_Abstract) {
-                // function/method call
-                $ns   = $dispatchable->getNamespace();
-                $name = $dispatchable->getName();
-                $name = empty($ns) ? $name : $ns . '.' . $name;
-
-                if (isset($table[$name])) {
-                    throw new Zend_XmlRpc_Server_Exception('Duplicate method registered: ' . $name);
-                }
-                $table[$name] = $dispatchable;
-                $this->_fixTypes($dispatchable);
-
-                continue;
-            }
-
-            if ($dispatchable instanceof Zend_Server_Reflection_Class) {
-                foreach ($dispatchable->getMethods() as $method) {
-                    $ns   = $method->getNamespace();
-                    $name = $method->getName();
-                    $name = empty($ns) ? $name : $ns . '.' . $name;
-
-                    if (isset($table[$name])) {
-                        throw new Zend_XmlRpc_Server_Exception('Duplicate method registered: ' . $name);
-                    }
-                    $table[$name] = $method;
-                    $this->_fixTypes($method);
-                    continue;
-                }
-            }
-        }
-
-        $this->_table = $table;
-    }
-
-    /**
-     * Set encoding
-     *
-     * @param string $encoding
-     * @return Zend_XmlRpc_Server
-     */
-    public function setEncoding($encoding)
-    {
-        $this->_encoding = $encoding;
-        return $this;
-    }
-
-    /**
-     * Retrieve current encoding
-     *
-     * @return string
-     */
-    public function getEncoding()
-    {
-        return $this->_encoding;
+        return call_user_func_array(array($system, $method), $params);
     }
 
     /**
@@ -310,6 +233,7 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
     public function addFunction($function, $namespace = '')
     {
         if (!is_string($function) && !is_array($function)) {
+            require_once 'Zend/XmlRpc/Server/Exception.php';
             throw new Zend_XmlRpc_Server_Exception('Unable to attach function; invalid', 611);
         }
 
@@ -322,57 +246,12 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
         $function = (array) $function;
         foreach ($function as $func) {
             if (!is_string($func) || !function_exists($func)) {
+                require_once 'Zend/XmlRpc/Server/Exception.php';
                 throw new Zend_XmlRpc_Server_Exception('Unable to attach function; invalid', 611);
             }
-            $this->_methods[] = Zend_Server_Reflection::reflectFunction($func, $argv, $namespace);
+            $reflection = Zend_Server_Reflection::reflectFunction($func, $argv, $namespace);
+            $this->_buildSignature($reflection);
         }
-
-        $this->_buildDispatchTable();
-    }
-
-    /**
-     * Load methods as returned from {@link getFunctions}
-     *
-     * Typically, you will not use this method; it will be called using the
-     * results pulled from {@link Zend_XmlRpc_Server_Cache::get()}.
-     *
-     * @param  array $definition
-     * @return void
-     * @throws Zend_XmlRpc_Server_Exception on invalid input
-     */
-    public function loadFunctions($definition)
-    {
-        if (!is_array($definition)) {
-            throw new Zend_XmlRpc_Server_Exception('Unable to load array; not an array', 612);
-        }
-
-        foreach ($definition as $key => $value) {
-            if (!$value instanceof Zend_Server_Reflection_Function_Abstract
-                && !$value instanceof Zend_Server_Reflection_Class)
-            {
-                throw new Zend_XmlRpc_Server_Exception('One or more method records are corrupt or otherwise unusable', 613);
-            }
-
-            if ($value->system) {
-                unset($definition[$key]);
-            }
-        }
-
-        foreach ($definition as $dispatchable) {
-            $this->_methods[] = $dispatchable;
-        }
-
-        $this->_buildDispatchTable();
-    }
-
-    /**
-     * Do nothing; persistence is handled via {@link Zend_XmlRpc_Server_Cache}
-     *
-     * @param  mixed $mode
-     * @return void
-     */
-    public function setPersistence($mode)
-    {
     }
 
     /**
@@ -395,52 +274,20 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
     public function setClass($class, $namespace = '', $argv = null)
     {
         if (is_string($class) && !class_exists($class)) {
-            if (!class_exists($class)) {
-                throw new Zend_XmlRpc_Server_Exception('Invalid method class', 610);
-            }
+            require_once 'Zend/XmlRpc/Server/Exception.php';
+            throw new Zend_XmlRpc_Server_Exception('Invalid method class', 610);
         }
 
         $argv = null;
-        if (3 < func_num_args()) {
+        if (2 < func_num_args()) {
             $argv = func_get_args();
-            $argv = array_slice($argv, 3);
+            $argv = array_slice($argv, 2);
         }
 
-        $this->_methods[] = Zend_Server_Reflection::reflectClass($class, $argv, $namespace);
-        $this->_buildDispatchTable();
-    }
-
-    /**
-     * Set the request object
-     *
-     * @param string|Zend_XmlRpc_Request $request
-     * @return Zend_XmlRpc_Server
-     * @throws Zend_XmlRpc_Server_Exception on invalid request class or object
-     */
-    public function setRequest($request)
-    {
-        if (is_string($request) && class_exists($request)) {
-            $request = new $request();
-            if (!$request instanceof Zend_XmlRpc_Request) {
-                throw new Zend_XmlRpc_Server_Exception('Invalid request class');
-            }
-            $request->setEncoding($this->getEncoding());
-        } elseif (!$request instanceof Zend_XmlRpc_Request) {
-            throw new Zend_XmlRpc_Server_Exception('Invalid request object');
+        $dispatchable = Zend_Server_Reflection::reflectClass($class, $argv, $namespace);
+        foreach ($dispatchable->getMethods() as $reflection) {
+            $this->_buildSignature($reflection, $class);
         }
-
-        $this->_request = $request;
-        return $this;
-    }
-
-    /**
-     * Return currently registered request object
-     *
-     * @return null|Zend_XmlRpc_Request
-     */
-    public function getRequest()
-    {
-        return $this->_request;
     }
 
     /**
@@ -455,101 +302,15 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
         if (!$fault instanceof Exception) {
             $fault = (string) $fault;
             if (empty($fault)) {
-                $fault = 'Unknown error';
+                $fault = 'Unknown Error';
             }
+            require_once 'Zend/XmlRpc/Server/Exception.php';
             $fault = new Zend_XmlRpc_Server_Exception($fault, $code);
         }
 
         return Zend_XmlRpc_Server_Fault::getInstance($fault);
     }
 
-    /**
-     * Handle an xmlrpc call (actual work)
-     *
-     * @param Zend_XmlRpc_Request $request
-     * @return Zend_XmlRpc_Response
-     * @throws Zend_XmlRpcServer_Exception|Exception
-     * Zend_XmlRpcServer_Exceptions are thrown for internal errors; otherwise,
-     * any other exception may be thrown by the callback
-     */
-    protected function _handle(Zend_XmlRpc_Request $request)
-    {
-        $method = $request->getMethod();
-
-        // Check for valid method
-        if (!isset($this->_table[$method])) {
-            throw new Zend_XmlRpc_Server_Exception('Method "' . $method . '" does not exist', 620);
-        }
-
-        $info     = $this->_table[$method];
-        $params   = $request->getParams();
-        $argv     = $info->getInvokeArguments();
-        if (0 < count($argv)) {
-            $params = array_merge($params, $argv);
-        }
-
-        // Check calling parameters against signatures
-        $matched    = false;
-        $sigCalled  = $request->getTypes();
-
-        $sigLength  = count($sigCalled);
-        $paramsLen  = count($params);
-        if ($sigLength < $paramsLen) {
-            for ($i = $sigLength; $i < $paramsLen; ++$i) {
-                $xmlRpcValue = Zend_XmlRpc_Value::getXmlRpcValue($params[$i]);
-                $sigCalled[] = $xmlRpcValue->getType();
-            }
-        }
-
-        $signatures = $info->getPrototypes();
-        foreach ($signatures as $signature) {
-            $sigParams = $signature->getParameters();
-            $tmpParams = array();
-            foreach ($sigParams as $param) {
-                $tmpParams[] = $param->getType();
-            }
-            if ($sigCalled === $tmpParams) {
-                $matched = true;
-                break;
-            }
-        }
-        if (!$matched) {
-            throw new Zend_XmlRpc_Server_Exception('Calling parameters do not match signature', 623);
-        }
-
-        if ($info instanceof Zend_Server_Reflection_Function) {
-            $func = $info->getName();
-            $return = call_user_func_array($func, $params);
-        } elseif (($info instanceof Zend_Server_Reflection_Method) && $info->system) {
-            // System methods
-            $return = $info->invokeArgs($this, $params);
-        } elseif ($info instanceof Zend_Server_Reflection_Method) {
-            // Get class
-            $class = $info->getDeclaringClass()->getName();
-
-            if ('static' == $info->isStatic()) {
-                // for some reason, invokeArgs() does not work the same as
-                // invoke(), and expects the first argument to be an object.
-                // So, using a callback if the method is static.
-                $return = call_user_func_array(array($class, $info->getName()), $params);
-            } else {
-                // Object methods
-                try {
-                    $object = $info->getDeclaringClass()->newInstance();
-                } catch (Exception $e) {
-                    throw new Zend_XmlRpc_Server_Exception('Error instantiating class ' . $class . ' to invoke method ' . $info->getName(), 621);
-                }
-
-                $return = $info->invokeArgs($object, $params);
-            }
-        } else {
-            throw new Zend_XmlRpc_Server_Exception('Method missing implementation ' . get_class($info), 622);
-        }
-
-        $response = new ReflectionClass($this->_responseClass);
-        return $response->newInstance($return);
-    }
-
     /**
      * Handle an xmlrpc call
      *
@@ -585,6 +346,111 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
         return $response;
     }
 
+    /**
+     * Load methods as returned from {@link getFunctions}
+     *
+     * Typically, you will not use this method; it will be called using the
+     * results pulled from {@link Zend_XmlRpc_Server_Cache::get()}.
+     *
+     * @param  array|Zend_Server_Definition $definition
+     * @return void
+     * @throws Zend_XmlRpc_Server_Exception on invalid input
+     */
+    public function loadFunctions($definition)
+    {
+        if (!is_array($definition) && (!$definition instanceof Zend_Server_Definition)) {
+            if (is_object($definition)) {
+                $type = get_class($definition);
+            } else {
+                $type = gettype($definition);
+            }
+            require_once 'Zend/XmlRpc/Server/Exception.php';
+            throw new Zend_XmlRpc_Server_Exception('Unable to load server definition; must be an array or Zend_Server_Definition, received ' . $type, 612);
+        }
+
+        $this->_table->clearMethods();
+        $this->_registerSystemMethods();
+
+        if ($definition instanceof Zend_Server_Definition) {
+            $definition = $definition->getMethods();
+        }
+
+        foreach ($definition as $key => $method) {
+            if ('system.' == substr($key, 0, 7)) {
+                continue;
+            }
+            $this->_table->addMethod($method, $key);
+        }
+    }
+
+    /**
+     * Set encoding
+     *
+     * @param string $encoding
+     * @return Zend_XmlRpc_Server
+     */
+    public function setEncoding($encoding)
+    {
+        $this->_encoding = $encoding;
+        Zend_XmlRpc_Value::setEncoding($encoding);
+        return $this;
+    }
+
+    /**
+     * Retrieve current encoding
+     *
+     * @return string
+     */
+    public function getEncoding()
+    {
+        return $this->_encoding;
+    }
+
+    /**
+     * Do nothing; persistence is handled via {@link Zend_XmlRpc_Server_Cache}
+     *
+     * @param  mixed $mode
+     * @return void
+     */
+    public function setPersistence($mode)
+    {
+    }
+
+    /**
+     * Set the request object
+     *
+     * @param string|Zend_XmlRpc_Request $request
+     * @return Zend_XmlRpc_Server
+     * @throws Zend_XmlRpc_Server_Exception on invalid request class or object
+     */
+    public function setRequest($request)
+    {
+        if (is_string($request) && class_exists($request)) {
+            $request = new $request();
+            if (!$request instanceof Zend_XmlRpc_Request) {
+                require_once 'Zend/XmlRpc/Server/Exception.php';
+                throw new Zend_XmlRpc_Server_Exception('Invalid request class');
+            }
+            $request->setEncoding($this->getEncoding());
+        } elseif (!$request instanceof Zend_XmlRpc_Request) {
+            require_once 'Zend/XmlRpc/Server/Exception.php';
+            throw new Zend_XmlRpc_Server_Exception('Invalid request object');
+        }
+
+        $this->_request = $request;
+        return $this;
+    }
+
+    /**
+     * Return currently registered request object
+     *
+     * @return null|Zend_XmlRpc_Request
+     */
+    public function getRequest()
+    {
+        return $this->_request;
+    }
+
     /**
      * Set the class to use for the response
      *
@@ -593,15 +459,34 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
      */
     public function setResponseClass($class)
     {
-        if (class_exists($class)) {
-            $reflection = new ReflectionClass($class);
-            if ($reflection->isSubclassOf(new ReflectionClass('Zend_XmlRpc_Response'))) {
-                $this->_responseClass = $class;
-                return true;
-            }
-        }
+        if (!class_exists($class) or
+            ($c = new ReflectionClass($class) and !$c->isSubclassOf('Zend_XmlRpc_Response'))) {
 
-        return false;
+            require_once 'Zend/XmlRpc/Server/Exception.php';
+            throw new Zend_XmlRpc_Server_Exception('Invalid response class');
+        }
+        $this->_responseClass = $class;
+        return true;
+    }
+
+    /**
+     * Retrieve current response class
+     *
+     * @return string
+     */
+    public function getResponseClass()
+    {
+        return $this->_responseClass;
+    }
+
+    /**
+     * Retrieve dispatch table
+     *
+     * @return array
+     */
+    public function getDispatchTable()
+    {
+        return $this->_table;
     }
 
     /**
@@ -614,129 +499,117 @@ class Zend_XmlRpc_Server implements Zend_Server_Interface
      */
     public function getFunctions()
     {
-        $return = array();
-        foreach ($this->_methods as $method) {
-            if ($method instanceof Zend_Server_Reflection_Class
-                && ($method->system))
-            {
-                continue;
-            }
+        return $this->_table->toArray();
+    }
 
-            $return[] = $method;
+    /**
+     * Retrieve system object
+     *
+     * @return Zend_XmlRpc_Server_System
+     */
+    public function getSystem()
+    {
+        return $this->_system;
+    }
+
+    /**
+     * Send arguments to all methods?
+     *
+     * If setClass() is used to add classes to the server, this flag defined
+     * how to handle arguments. If set to true, all methods including constructor
+     * will receive the arguments. If set to false, only constructor will receive the
+     * arguments
+     */
+    public function sendArgumentsToAllMethods($flag = null)
+    {
+        if ($flag === null) {
+            return $this->_sendArgumentsToAllMethods;
         }
 
-        return $return;
+        $this->_sendArgumentsToAllMethods = (bool)$flag;
+        return $this;
     }
 
     /**
-     * List all available XMLRPC methods
+     * Map PHP type to XML-RPC type
      *
-     * Returns an array of methods.
-     *
-     * @return array
-     */
-    public function listMethods()
-    {
-        return array_keys($this->_table);
-    }
-
-    /**
-     * Display help message for an XMLRPC method
-     *
-     * @param string $method
+     * @param  string $type
      * @return string
      */
-    public function methodHelp($method)
+    protected function _fixType($type)
     {
-        if (!isset($this->_table[$method])) {
-            throw new Zend_Server_Exception('Method "' . $method . '"does not exist', 640);
+        if (isset($this->_typeMap[$type])) {
+            return $this->_typeMap[$type];
         }
-
-        return $this->_table[$method]->getDescription();
+        return 'void';
     }
 
     /**
-     * Return a method signature
+     * Handle an xmlrpc call (actual work)
      *
-     * @param string $method
-     * @return array
+     * @param Zend_XmlRpc_Request $request
+     * @return Zend_XmlRpc_Response
+     * @throws Zend_XmlRpcServer_Exception|Exception
+     * Zend_XmlRpcServer_Exceptions are thrown for internal errors; otherwise,
+     * any other exception may be thrown by the callback
      */
-    public function methodSignature($method)
+    protected function _handle(Zend_XmlRpc_Request $request)
     {
-        if (!isset($this->_table[$method])) {
-            throw new Zend_Server_Exception('Method "' . $method . '"does not exist', 640);
-        }
-        $prototypes = $this->_table[$method]->getPrototypes();
+        $method = $request->getMethod();
 
-        $signatures = array();
-        foreach ($prototypes as $prototype) {
-            $signature = array($prototype->getReturnType());
-            foreach ($prototype->getParameters() as $parameter) {
-                $signature[] = $parameter->getType();
+        // Check for valid method
+        if (!$this->_table->hasMethod($method)) {
+            require_once 'Zend/XmlRpc/Server/Exception.php';
+            throw new Zend_XmlRpc_Server_Exception('Method "' . $method . '" does not exist', 620);
+        }
+
+        $info     = $this->_table->getMethod($method);
+        $params   = $request->getParams();
+        $argv     = $info->getInvokeArguments();
+        if (0 < count($argv) and $this->sendArgumentsToAllMethods()) {
+            $params = array_merge($params, $argv);
+        }
+
+        // Check calling parameters against signatures
+        $matched    = false;
+        $sigCalled  = $request->getTypes();
+
+        $sigLength  = count($sigCalled);
+        $paramsLen  = count($params);
+        if ($sigLength < $paramsLen) {
+            for ($i = $sigLength; $i < $paramsLen; ++$i) {
+                $xmlRpcValue = Zend_XmlRpc_Value::getXmlRpcValue($params[$i]);
+                $sigCalled[] = $xmlRpcValue->getType();
             }
-            $signatures[] = $signature;
         }
 
-        return $signatures;
+        $signatures = $info->getPrototypes();
+        foreach ($signatures as $signature) {
+            $sigParams = $signature->getParameters();
+            if ($sigCalled === $sigParams) {
+                $matched = true;
+                break;
+            }
+        }
+        if (!$matched) {
+            require_once 'Zend/XmlRpc/Server/Exception.php';
+            throw new Zend_XmlRpc_Server_Exception('Calling parameters do not match signature', 623);
+        }
+
+        $return        = $this->_dispatch($info, $params);
+        $responseClass = $this->getResponseClass();
+        return new $responseClass($return);
     }
 
     /**
-     * Multicall - boxcar feature of XML-RPC for calling multiple methods
-     * in a single request.
+     * Register system methods with the server
      *
-     * Expects a an array of structs representing method calls, each element
-     * having the keys:
-     * - methodName
-     * - params
-     *
-     * Returns an array of responses, one for each method called, with the value
-     * returned by the method. If an error occurs for a given method, returns a
-     * struct with a fault response.
-     *
-     * @see http://www.xmlrpc.com/discuss/msgReader$1208
-     * @param  array $methods
-     * @return array
+     * @return void
      */
-    public function multicall($methods)
+    protected function _registerSystemMethods()
     {
-        $responses = array();
-        foreach ($methods as $method) {
-            $fault = false;
-            if (!is_array($method)) {
-                $fault = $this->fault('system.multicall expects each method to be a struct', 601);
-            } elseif (!isset($method['methodName'])) {
-                $fault = $this->fault('Missing methodName', 602);
-            } elseif (!isset($method['params'])) {
-                $fault = $this->fault('Missing params', 603);
-            } elseif (!is_array($method['params'])) {
-                $fault = $this->fault('Params must be an array', 604);
-            } else {
-                if ('system.multicall' == $method['methodName']) {
-                    // don't allow recursive calls to multicall
-                    $fault = $this->fault('Recursive system.multicall forbidden', 605);
-                }
-            }
-
-            if (!$fault) {
-                try {
-                    $request = new Zend_XmlRpc_Request();
-                    $request->setMethod($method['methodName']);
-                    $request->setParams($method['params']);
-                    $response = $this->_handle($request);
-                    $responses[] = $response->getReturnValue();
-                } catch (Exception $e) {
-                    $fault = $this->fault($e);
-                }
-            }
-
-            if ($fault) {
-                $responses[] = array(
-                    'faultCode'   => $fault->getCode(),
-                    'faultString' => $fault->getMessage()
-                );
-            }
-        }
-
-        return $responses;
+        $system = new Zend_XmlRpc_Server_System($this);
+        $this->_system = $system;
+        $this->setClass($system, 'system');
     }
 }
